# coding=utf-8
#
# Copyright 2017 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

from f5.bigip.resource import Collection
from f5.bigip.resource import OrganizingCollection
from f5.bigip.resource import TaskResource
from f5.sdk_exception import MissingRequiredCreationParameter
from f5.sdk_exception import UnsupportedOperation


class Iapp(OrganizingCollection):
    def __init__(self, shared):
        super(Iapp, self).__init__(shared)
        self._meta_data['allowed_lazy_attributes'] = [
            Package_Management_Tasks_s
        ]


class Package_Management_Tasks_s(Collection):
    def __init__(self, iapp):
        super(Package_Management_Tasks_s, self).__init__(iapp)
        self._meta_data['object_has_stats'] = False
        self._meta_data['required_json_kind'] = \
            'shared:iapp:package-management-tasks:iapppackagemanagementcollectionstate'  # NOQA
        self._meta_data['allowed_lazy_attributes'] = [Package_Management_Task]
        self._meta_data['attribute_registry'] = {
            'shared:iapp:package-management-tasks:iapppackagemanagementtaskstate': Package_Management_Task  # NOQA
        }


class Package_Management_Task(TaskResource):
    def __init__(self, package_management_tasks):
        super(Package_Management_Task, self).__init__(package_management_tasks)
        self._meta_data['required_json_kind'] = \
            'shared:iapp:package-management-tasks:iapppackagemanagementtaskstate'  # NOQA
        self._meta_data['required_creation_parameters'] = {'operation'}

    def create(self, **kwargs):
        if 'operation' not in kwargs:
            error_message = "Missing required params: ['operation']"
            raise MissingRequiredCreationParameter(error_message)

        if kwargs['operation'] == 'INSTALL':
            if 'packageFilePath' not in kwargs:
                error_message = "Missing required params: ['packageFilePath']"
                raise MissingRequiredCreationParameter(error_message)
        return self._create(**kwargs)

    def update(self, **kwargs):
        raise UnsupportedOperation(
            "%s does not support the update method" % self.__class__.__name__
        )

    def cancel(self, **kwargs):
        self._cancel(**kwargs)

    def _cancel(self, **kwargs):
        params = dict(
            status='CANCEL_REQUESTED'
        )
        self.modify(**params)
        if self.status == 'CANCEL_REQUESTED':
            self.__dict__.update({'canceled': True})



# coding=utf-8
#
#  Copyright 2014-2016 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""BIG-IP® Local Traffic Manager (LTM) policy module.

REST URI
    ``http://localhost/mgmt/tm/ltm/policy``

GUI Path
    ``Local Traffic --> policy``

REST Kind
    ``tm:ltm:policy:*``
"""

from f5.bigip.mixins import CheckExistenceMixin
from f5.bigip.resource import Collection
from f5.bigip.resource import Resource
from f5.sdk_exception import DraftPolicyNotSupportedInTMOSVersion
from f5.sdk_exception import MissingRequiredCreationParameter
from f5.sdk_exception import NonExtantPolicyRule
from f5.sdk_exception import OperationNotSupportedOnPublishedPolicy

from distutils.version import LooseVersion
import logging

logger = logging.getLogger(__name__)


class Policys(Collection):
    """BIG-IP® LTM policy collection."""
    def __init__(self, ltm):
        super(Policys, self).__init__(ltm)
        self._meta_data['allowed_lazy_attributes'] = [Policy]
        self._meta_data['attribute_registry'] =\
            {'tm:ltm:policy:policystate': Policy}


class Policy(Resource):
    """BIG-IP® LTM policy resource."""
    def __init__(self, policy_s):
        super(Policy, self).__init__(policy_s)
        self._meta_data['allowed_lazy_attributes'] = [Rules_s]
        self._meta_data['required_json_kind'] = 'tm:ltm:policy:policystate'
        self._meta_data['required_creation_parameters'].update(('strategy',))
        temp = {'tm:ltm:policy:rules:rulescollectionstate': Rules_s}
        self._meta_data['attribute_registry'] = temp
        self._meta_data['optional_parameters'] = {'rules': ['description']}

    def _filter_version_specific_options(self, tmos_ver, **kwargs):
        '''Filter version-specific optional parameters

        Some optional parameters only exist in v12.1.0 and greater,
        filter these out for earlier versions to allow backward comatibility.
        '''

        if LooseVersion(tmos_ver) < LooseVersion('12.1.0'):
            for k, parms in self._meta_data['optional_parameters'].items():
                for r in kwargs.get(k, []):
                    for parm in parms:
                        value = r.pop(parm, None)
                        if value is not None:
                            logger.info(
                                "Policy parameter %s:%s is invalid for v%s",
                                k, parm, tmos_ver)

    def _create(self, **kwargs):
        '''Allow creation of draft policy and ability to publish a draft

        Draft policies only exist in 12.1.0 and greater versions of TMOS.
        But there must be a method to create a draft, then publish it.

        :raises: MissingRequiredCreationParameter
        '''

        tmos_ver = self._meta_data['bigip']._meta_data['tmos_version']
        legacy = kwargs.pop('legacy', False)
        publish = kwargs.pop('publish', False)
        self._filter_version_specific_options(tmos_ver, **kwargs)
        if LooseVersion(tmos_ver) < LooseVersion('12.1.0'):
            return super(Policy, self)._create(**kwargs)
        else:
            if legacy:
                return super(Policy, self)._create(legacy=True, **kwargs)
            else:
                if 'subPath' not in kwargs:
                    msg = "The keyword 'subPath' must be specified when " \
                        "creating draft policy in TMOS versions >= 12.1.0. " \
                        "Try and specify subPath as 'Drafts'."
                    raise MissingRequiredCreationParameter(msg)
                self = super(Policy, self)._create(**kwargs)
                if publish:
                    self.publish()
                return self

    def _modify(self, **patch):
        '''Modify only draft or legacy policies

        Published policies cannot be modified
        :raises: OperationNotSupportedOnPublishedPolicy
        '''

        legacy = patch.pop('legacy', False)
        tmos_ver = self._meta_data['bigip']._meta_data['tmos_version']
        self._filter_version_specific_options(tmos_ver, **patch)
        if 'Drafts' not in self._meta_data['uri'] and \
                LooseVersion(tmos_ver) >= LooseVersion('12.1.0') and \
                not legacy:
            msg = 'Modify operation not allowed on a published policy.'
            raise OperationNotSupportedOnPublishedPolicy(msg)
        super(Policy, self)._modify(**patch)

    def _update(self, **kwargs):
        '''Update only draft or legacy policies

        Published policies cannot be updated
        :raises: OperationNotSupportedOnPublishedPolicy
        '''

        legacy = kwargs.pop('legacy', False)
        tmos_ver = self._meta_data['bigip']._meta_data['tmos_version']
        self._filter_version_specific_options(tmos_ver, **kwargs)
        if 'Drafts' not in self._meta_data['uri'] and \
                LooseVersion(tmos_ver) >= LooseVersion('12.1.0') and \
                not legacy:
            msg = 'Update operation not allowed on a published policy.'
            raise OperationNotSupportedOnPublishedPolicy(msg)
        super(Policy, self)._update(**kwargs)

    def publish(self, **kwargs):
        '''Publishing a draft policy is only applicable in TMOS 12.1 and up.

        This operation updates the meta_data['uri'] of the existing object
        and effectively moves a draft into a published state on the device.
        The self object is also updated with the response from a GET to the
        device.

        :raises: PolicyNotDraft
        '''

        assert 'Drafts' in self._meta_data['uri']
        assert self.status.lower() == 'draft'
        base_uri = self._meta_data['container']._meta_data['uri']
        requests_params = self._handle_requests_params(kwargs)
        session = self._meta_data['bigip']._meta_data['icr_session']
        if 'command' not in kwargs:
            kwargs['command'] = 'publish'
        if 'Drafts' not in self.name:
            kwargs['name'] = self.fullPath
        session.post(base_uri, json=kwargs, **requests_params)
        get_kwargs = {
            'name': self.name, 'partition': self.partition,
            'uri_as_parts': True
        }
        response = session.get(base_uri, **get_kwargs)
        json_data = response.json()
        self._local_update(json_data)
        self._activate_URI(json_data['selfLink'])

    def draft(self, **kwargs):
        '''Allows for easily re-drafting a policy

        After a policy has been created, it was not previously possible
        to re-draft the published policy. This method makes it possible
        for a user with existing, published, policies to create drafts
        from them so that they are modifiable.
        See https://github.com/F5Networks/f5-common-python/pull/1099

        :param kwargs:
        :return:
        '''
        tmos_ver = self._meta_data['bigip']._meta_data['tmos_version']
        legacy = kwargs.pop('legacy', False)
        if LooseVersion(tmos_ver) < LooseVersion('12.1.0') or legacy:
            raise DraftPolicyNotSupportedInTMOSVersion(
                "Drafting on this version of BIG-IP is not supported"
            )
        kwargs = dict(
            createDraft=True
        )
        super(Policy, self)._modify(**kwargs)

        get_kwargs = {
            'name': self.name,
            'partition': self.partition,
            'uri_as_parts': True,
            'subPath': 'Drafts'
        }
        base_uri = self._meta_data['container']._meta_data['uri']
        session = self._meta_data['bigip']._meta_data['icr_session']
        response = session.get(base_uri, **get_kwargs)
        json_data = response.json()
        self._local_update(json_data)
        self._activate_URI(json_data['selfLink'])


class Rules_s(Collection):
    """BIG-IP® LTM policy rules sub-collection."""
    def __init__(self, policy):
        super(Rules_s, self).__init__(policy)
        self._meta_data['attribute_registry'] =\
            {'tm:ltm:policy:rules:rulesstate': Rules}
        self._meta_data['required_json_kind'] =\
            'tm:ltm:policy:rules:rulescollectionstate'
        self._meta_data['allowed_lazy_attributes'] = [Rules]


class Rules(Resource, CheckExistenceMixin):
    """BIG-IP® LTM policy rules sub-collection resource."""
    def __init__(self, rules_s):
        super(Rules, self).__init__(rules_s)
        self._meta_data['required_json_kind'] =\
            'tm:ltm:policy:rules:rulesstate'
        temp = {'tm:ltm:policy:rules:actions:actionscollectionstate':
                Actions_s,
                'tm:ltm:policy:rules:conditions:conditionscollectionstate':
                Conditions_s}
        self._meta_data['attribute_registry'] = temp

    def _load(self, **kwargs):
        """Must check if rule actually exists before proceeding with load."""

        if self._check_existence_by_collection(
                self._meta_data['container'], kwargs['name']):
            return super(Rules, self)._load(**kwargs)
        msg = 'The rule named, {}, does not exist on the device.'.format(
            kwargs['name'])
        raise NonExtantPolicyRule(msg)

    def exists(self, **kwargs):
        """Check rule existence on device."""

        return self._check_existence_by_collection(
            self._meta_data['container'], kwargs['name'])


class Actions_s(Collection):
    """BIG-IP® LTM policy actions sub-collection."""
    def __init__(self, rules):
        super(Actions_s, self).__init__(rules)
        self._meta_data['required_json_kind'] =\
            'tm:ltm:policy:rules:actions:actionscollectionstate'
        self._meta_data['allowed_lazy_attributes'] = [Actions]
        self._meta_data['attribute_registry'] =\
            {'tm:ltm:policy:rules:actions:actionsstate': Actions}


class Actions(Resource):
    """BIG-IP® LTM policy actions sub-collection resource."""
    def __init__(self, actions_s):
        super(Actions, self).__init__(actions_s)
        self._meta_data['required_json_kind'] =\
            'tm:ltm:policy:rules:actions:actionsstate'


class Conditions_s(Collection):
    """BIG-IP® LTM policy conditions sub-collection."""
    def __init__(self, rules):
        super(Conditions_s, self).__init__(rules)
        self._meta_data['required_json_kind'] =\
            'tm:ltm:policy:rules:conditions:conditionscollectionstate'
        self._meta_data['allowed_lazy_attributes'] = [Conditions]
        self._meta_data['attribute_registry'] =\
            {'tm:ltm:policy:rules:conditions:conditionsstate': Conditions}


class Conditions(Resource):
    """BIG-IP® LTM policy conditions sub-collection resource."""
    def __init__(self, conditions_s):
        super(Conditions, self).__init__(conditions_s)
        self._meta_data['required_json_kind'] =\
            'tm:ltm:policy:rules:conditions:conditionsstate'



# coding=utf-8
#
# Copyright 2014-2016 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""BIG-IP® Local Traffic Manager (LTM) Snat module.

REST URI
    ``http://localhost/mgmt/tm/ltm/snat``

GUI Path
    ``Local Traffic --> Snat``

REST Kind
    ``tm:ltm:snat:*``
"""

from f5.bigip.resource import Collection
from f5.bigip.resource import Resource
from f5.sdk_exception import MissingRequiredCreationParameter

from six import iterkeys


# ToDo This is a prime example of duplication, needs to be removed
class RequireOneOf(MissingRequiredCreationParameter):
    pass


class Snats(Collection):
    """BIG-IP® LTM Snat collection"""
    def __init__(self, ltm):
        super(Snats, self).__init__(ltm)
        self._meta_data['allowed_lazy_attributes'] = [Snat]
        self._meta_data['attribute_registry'] =\
            {'tm:ltm:snat:snatstate': Snat}


class Snat(Resource):
    """BIG-IP® LTM Snat resource"""
    def __init__(self, snat_s):
        '''This represents a Snat.

        "origins" is our first example of a dict attribute, it appears to
        behave as expected.
        '''
        super(Snat, self).__init__(snat_s)
        self._meta_data['required_json_kind'] = 'tm:ltm:snat:snatstate'
        self._meta_data['required_creation_parameters'].update(
            ('partition', 'origins'))

    def create(self, **kwargs):
        """Call this to create a new snat on the BIG-IP®.

        Uses HTTP POST to 'containing' URI to create a service associated with
        a new URI on the device.

        Note this is the one of two fundamental Resource operations that
        returns a different uri (in the returned object) than the uri the
        operation was called on.  The returned uri can be accessed as
        Object.selfLink, the actual uri used by REST operations on the object
        is Object._meta_data['uri'].  The _meta_data['uri'] is the same as
        Object.selfLink with the substring 'localhost' replaced with the value
        of Object._meta_data['BIG-IP']._meta_data['hostname'], and without
        query args, or hash fragments.

        The following is done prior to the POST
        * Ensures that one of ``automap``, ``snatpool``, ``translation``
          parameter is passed in.

        :param kwargs: All the key-values needed to create the resource
        :returns: An instance of the Python object that represents the device's
        uri-published resource.  The uri of the resource is part of the
        object's _meta_data.
        """
        required_one_of = ['automap', 'snatpool', 'translation']
        has_any = [x for x in iterkeys(kwargs) if x in required_one_of]

        # The rules for SNAT is that you must provide one, and only one,
        # of the `required_one_of` values above.
        if len(has_any) == 1:
            return self._create(**kwargs)

        raise RequireOneOf(
            "Creation requires one of the provided {0}".format(
                ', '.join(required_one_of)
            )
        )



# coding=utf-8
#
#  Copyright 2014-2016 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""BIG-IP® LTM profile submodule.

REST URI
    ``http://localhost/mgmt/tm/ltm/profile/``

GUI Path
    ``Local Traffic --> Profiles``

REST Kind
    ``tm:ltm:profile*``
"""


from f5.bigip.resource import Collection
from f5.bigip.resource import OrganizingCollection
from f5.bigip.resource import Resource
from f5.sdk_exception import MissingUpdateParameter
from f5.sdk_exception import UnsupportedOperation


class Profile(OrganizingCollection):
    def __init__(self, ltm):
        super(Profile, self).__init__(ltm)
        self._meta_data['allowed_lazy_attributes'] = [
            Analytics_s,
            Certificate_Authoritys,
            Classifications,
            Client_Ldaps,
            Client_Ssls,
            Dhcpv4s,
            Dhcpv6s,
            Diameters,
            Dns_s,
            Dns_Loggings,
            Fasthttps,
            Fastl4s,
            Fixs,
            Ftps,
            Gtps,
            Htmls,
            Https,
            Http_Compressions,
            Http2s,
            Icaps,
            Iiops,
            Ipothers,
            Ipsecalgs,
            Mblbs,
            Mssqls,
            Ntlms,
            Ocsp_Stapling_Params_s,
            One_Connects,
            Pcps,
            Pptps,
            Qoes,
            Radius_s,
            Ramcaches,
            Request_Adapts,
            Request_Logs,
            Response_Adapts,
            Rewrites,
            Rtsps,
            Sctps,
            Server_Ldaps,
            Server_Ssls,
            Sips,
            Smtps,
            Smtps_s,
            Socks_s,
            Spdys,
            Statistics_s,
            Streams,
            Tcps,
            Tftps,
            Udps,
            Wa_Caches,
            Web_Accelerations,
            Web_Securitys,
            Xmls]


class Client_Ssls(Collection):
    """BIG-IP® Client SSL profile collection."""
    def __init__(self, profile):
        super(Client_Ssls, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Client_Ssl]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:client-ssl:client-sslstate': Client_Ssl}


class Client_Ssl(Resource):
    def __init__(self, client_ssls):
        super(Client_Ssl, self).__init__(client_ssls)
        self._meta_data['required_json_kind'] =\
            'tm:ltm:profile:client-ssl:client-sslstate'


class Analytics_s(Collection):
    """BIG-IP® Analytics profile collection.

    .. note::
         Profile and sub-collections
         require AVR provisioned.
    """
    def __init__(self, profile):
        super(Analytics_s, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Analytics]
        self._meta_data['attribute_registry'] =\
            {'tm:ltm:profile:analytics:analyticsstate': Analytics}


class Analytics(Resource):
    """BIG-IP® Analytics profile resource."""
    def __init__(self, Analytics_s):
        super(Analytics, self).__init__(Analytics_s)
        self._meta_data['allowed_lazy_attributes'] = [Alerts_s,
                                                      Traffic_Captures]
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:analytics:analyticsstate'
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:analytics:alerts:alertscollectionstate': Alerts_s,
             'tm:ltm:profile:analytics:traffic-capture:\
             traffic-capturecollectionstate':
                 Traffic_Captures}


class Alerts_s(Collection):
    """BIG-IP® Alerts sub-collection."""
    def __init__(self, Analytics):
        super(Alerts_s, self).__init__(Analytics)
        self._meta_data['allowed_lazy_attributes'] = [Alerts]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:analytics:alerts:alertsstate': Alerts}


class Traffic_Captures(Collection):
    """BIG-IP® Traffic Capture sub-collection."""
    def __init__(self, Analytics):
        super(Traffic_Captures, self).__init__(Analytics)
        self._meta_data['allowed_lazy_attributes'] = [Traffic_Capture]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:analytics:traffic-capture:\
            traffic-capturestate': Traffic_Capture}


class Alerts(Resource):
    """BIG-IP® Alerts resource."""
    def __init__(self, Alerts_s):
        super(Alerts, self).__init__(Alerts_s)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:analytics:alerts:alertsstate'


class Traffic_Capture(Resource):
    """BIG-IP® Traffic Capture resource."""
    def __init__(self, Traffic_Captures):
        super(Traffic_Capture, self).__init__(Traffic_Captures)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:analytics:traffic-capture:traffic-capturestate'


class Certificate_Authoritys(Collection):
    """BIG-IP® Certificate Authority profile collection."""
    def __init__(self, profile):
        super(Certificate_Authoritys, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Certificate_Authority]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:certificate-authority:\
            certificate-authoritystate': Certificate_Authority}


class Certificate_Authority(Resource):
    """BIG-IP® Certificate Authority resource."""
    def __init__(self, Certificate_Authoritys):
        super(Certificate_Authority, self).__init__(Certificate_Authoritys)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:certificate-authority:certificate-authoritystate'


class Classifications(Collection):
    """BIG-IP® Classification profile collection."""
    def __init__(self, profile):
        super(Classifications, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Classification]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:classification:\
            classificationstate': Classification}


class Classification(Resource):
    """BIG-IP® Classification resource."""
    def __init__(self, Classifications):
        super(Classification, self).__init__(Classifications)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:classification:classificationstate'

    def create(self, **kwargs):
        """Create is not supported for Classification

        :raises: UnsupportedOperation
        """
        raise UnsupportedOperation(
            "%s does not support the update method" % self.__class__.__name__
        )

    def delete(self, **kwargs):
        """Delete is not supported for Classification

        :raises: UnsupportedOperation
        """
        raise UnsupportedOperation(
            "%s does not support the delete method" % self.__class__.__name__
        )


class Client_Ldaps(Collection):
    """BIG-IP® Client Ldap profile collection."""
    def __init__(self, profile):
        super(Client_Ldaps, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Client_Ldap]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:client-ldap:client-ldapstate': Client_Ldap}
        self._meta_data['minimum_version'] = '11.6.0'


class Client_Ldap(Resource):
    """BIG-IP® Client Ldap resource."""
    def __init__(self, Client_Ldaps):
        super(Client_Ldap, self).__init__(Client_Ldaps)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:client-ldap:client-ldapstate'


class Dhcpv4s(Collection):
    """BIG-IP® Dhcpv4 profile collection."""
    def __init__(self, profile):
        super(Dhcpv4s, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Dhcpv4]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:dhcpv4:dhcpv4state': Dhcpv4}
        self._meta_data['minimum_version'] = '11.6.0'


class Dhcpv4(Resource):
    """BIG-IP® Dhcpv4 resource."""
    def __init__(self, Dhcpv4s):
        super(Dhcpv4, self).__init__(Dhcpv4s)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:dhcpv4:dhcpv4state'


class Dhcpv6s(Collection):
    """BIG-IP® Dhcpv6 profile collection."""
    def __init__(self, profile):
        super(Dhcpv6s, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Dhcpv6]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:dhcpv6:dhcpv6state': Dhcpv6}
        self._meta_data['minimum_version'] = '11.6.0'


class Dhcpv6(Resource):
    """BIG-IP® Dhcpv6 resource."""
    def __init__(self, Dhcpv6s):
        super(Dhcpv6, self).__init__(Dhcpv6s)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:dhcpv6:dhcpv6state'


class Diameters(Collection):
    """BIG-IP® Diameter profile collection."""
    def __init__(self, profile):
        super(Diameters, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Diameter]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:diameter:diameterstate': Diameter}


class Diameter(Resource):
    """BIG-IP® Diameter resource."""
    def __init__(self, Diameters):
        super(Diameter, self).__init__(Diameters)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:diameter:diameterstate'


class Dns_s(Collection):
    """BIG-IP® DNS profile collection."""
    def __init__(self, profile):
        super(Dns_s, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Dns]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:dns:dnsstate': Dns}


class Dns(Resource):
    """BIG-IP® DNS resource."""
    def __init__(self, Dns_s):
        super(Dns, self).__init__(Dns_s)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:dns:dnsstate'


class Dns_Loggings(Collection):
    """BIG-IP® DNS Logging profile collection."""
    def __init__(self, profile):
        super(Dns_Loggings, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Dns_Logging]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:dns-logging:dns-loggingstate': Dns_Logging}


class Dns_Logging(Resource):
    """BIG-IP® DNS Logging resource."""
    def __init__(self, Dns_Loggings):
        super(Dns_Logging, self).__init__(Dns_Loggings)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:dns-logging:dns-loggingstate'
        self._meta_data['required_creation_parameters'].update(
            ('logPublisher',))


class Fasthttps(Collection):
    """BIG-IP® Fasthttp profile collection."""
    def __init__(self, profile):
        super(Fasthttps, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Fasthttp]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:fasthttp:fasthttpstate': Fasthttp}


class Fasthttp(Resource):
    """BIG-IP® Fasthttp resource."""
    def __init__(self, Fasthttps):
        super(Fasthttp, self).__init__(Fasthttps)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:fasthttp:fasthttpstate'


class Fastl4s(Collection):
    """BIG-IP® Fastl4 profile collection."""
    def __init__(self, profile):
        super(Fastl4s, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Fastl4]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:fastl4:fastl4state': Fastl4}


class Fastl4(Resource):
    """BIG-IP® Fastl4 resource."""
    def __init__(self, Fastl4s):
        super(Fastl4, self).__init__(Fastl4s)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:fastl4:fastl4state'


class Fixs(Collection):
    """BIG-IP® Fix profile collection."""
    def __init__(self, profile):
        super(Fixs, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Fix]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:fix:fixstate': Fix}


class Fix(Resource):
    """BIG-IP® Fix resource."""
    def __init__(self, Fixs):
        super(Fix, self).__init__(Fixs)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:fix:fixstate'


class Ftps(Collection):
    """BIG-IP® Ftp profile collection."""
    def __init__(self, profile):
        super(Ftps, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Ftp]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:ftp:ftpstate': Ftp}


class Ftp(Resource):
    """BIG-IP® Ftp resource."""
    def __init__(self, Ftps):
        super(Ftp, self).__init__(Ftps)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:ftp:ftpstate'


class Gtps(Collection):
    """BIG-IP® Gtp profile collection."""
    def __init__(self, profile):
        super(Gtps, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Gtp]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:gtp:gtpstate': Gtp}
        self._meta_data['minimum_version'] = '11.6.0'


class Gtp(Resource):
    """BIG-IP® Gtp resource."""
    def __init__(self, Gtps):
        super(Gtp, self).__init__(Gtps)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:gtp:gtpstate'


class Htmls(Collection):
    """BIG-IP® Html profile collection."""
    def __init__(self, profile):
        super(Htmls, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Html]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:html:htmlstate': Html}


class Html(Resource):
    """BIG-IP® Html resource."""
    def __init__(self, Htmls):
        super(Html, self).__init__(Htmls)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:html:htmlstate'


class Https(Collection):
    """BIG-IP® Http profile collection."""
    def __init__(self, profile):
        super(Https, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Http]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:http:httpstate': Http}


class Http(Resource):
    """BIG-IP® Http resource."""
    def __init__(self, Https):
        super(Http, self).__init__(Https)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:http:httpstate'


class Http_Compressions(Collection):
    """BIG-IP® Http_Compression profile collection."""
    def __init__(self, profile):
        super(Http_Compressions, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Http_Compression]
        temp = \
            {'tm:ltm:profile:http-compression:http-compressionstate':
                Http_Compression}
        self._meta_data['attribute_registry'] = temp


class Http_Compression(Resource):
    """BIG-IP® Http_Compression resource."""
    def __init__(self, Http_Compressions):
        super(Http_Compression, self).__init__(Http_Compressions)
        temp = \
            'tm:ltm:profile:http-compression:http-compressionstate'
        self._meta_data['required_json_kind'] = temp


class Http2s(Collection):
    """BIG-IP® Http2 profile collection."""
    def __init__(self, profile):
        super(Http2s, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Http2]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:http2:http2state': Http2}
        self._meta_data['minimum_version'] = '11.6.0'


class Http2(Resource):
    """BIG-IP® Http2 resource."""
    def __init__(self, Http2s):
        super(Http2, self).__init__(Http2s)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:http2:http2state'


class Icaps(Collection):
    """BIG-IP® Icap profile collection."""
    def __init__(self, profile):
        super(Icaps, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Icap]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:icap:icapstate': Icap}


class Icap(Resource):
    """BIG-IP® Icap resource."""
    def __init__(self, Icaps):
        super(Icap, self).__init__(Icaps)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:icap:icapstate'


class Iiops(Collection):
    """BIG-IP® Iiop profile collection."""
    def __init__(self, profile):
        super(Iiops, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Iiop]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:iiop:iiopstate': Iiop}
        self._meta_data['minimum_version'] = '12.0.0'


class Iiop(Resource):
    """BIG-IP® Iiop resource."""
    def __init__(self, Iiops):
        super(Iiop, self).__init__(Iiops)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:iiop:iiopstate'


class Ipothers(Collection):
    """BIG-IP® Ipother profile collection."""
    def __init__(self, profile):
        super(Ipothers, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Ipother]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:ipother:ipotherstate': Ipother}


class Ipother(Resource):
    """BIG-IP® Ipother resource."""
    def __init__(self, Ipothers):
        super(Ipother, self).__init__(Ipothers)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:ipother:ipotherstate'


class Ipsecalgs(Collection):
    """BIG-IP® IPsecALG profile collection."""
    def __init__(self, profile):
        super(Ipsecalgs, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Ipsecalg]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:ipsecalg:ipsecalgstate': Ipsecalg}
        self._meta_data['minimum_version'] = '13.0.0'


class Ipsecalg(Resource):
    """BIG-IP® IPsecALG resource."""
    def __init__(self, Ipsecalgs):
        super(Ipsecalg, self).__init__(Ipsecalgs)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:ipsecalg:ipsecalgstate'


class Mblbs(Collection):
    """BIG-IP® Mblb profile collection."""
    def __init__(self, profile):
        super(Mblbs, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Mblb]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:mblb:mblbstate': Mblb}


class Mblb(Resource):
    """BIG-IP® Mblb resource."""
    def __init__(self, Mblbs):
        super(Mblb, self).__init__(Mblbs)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:mblb:mblbstate'


class Mssqls(Collection):
    """BIG-IP® Mssql profile collection."""
    def __init__(self, profile):
        super(Mssqls, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Mssql]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:mssql:mssqlstate': Mssql}


class Mssql(Resource):
    """BIG-IP® Mssql resource."""
    def __init__(self, Mssqls):
        super(Mssql, self).__init__(Mssqls)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:mssql:mssqlstate'


class Ntlms(Collection):
    """BIG-IP® Ntlm profile collection."""
    def __init__(self, profile):
        super(Ntlms, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Ntlm]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:ntlm:ntlmstate': Ntlm}


class Ntlm(Resource):
    """BIG-IP® Ntlm resource."""
    def __init__(self, Ntlms):
        super(Ntlm, self).__init__(Ntlms)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:ntlm:ntlmstate'


class Ocsp_Stapling_Params_s(Collection):
    """BIG-IP® Ocsp_Stapling_Params profile collection."""
    def __init__(self, profile):
        super(Ocsp_Stapling_Params_s, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Ocsp_Stapling_Params]
        temp = \
            {'tm:ltm:profile:ocsp-stapling-params:ocsp-stapling-paramsstate':
                Ocsp_Stapling_Params}
        self._meta_data['attribute_registry'] = temp
        self._meta_data['minimum_version'] = '11.6.0'


class Ocsp_Stapling_Params(Resource):
    """BIG-IP® Ocsp_Stapling_Params resource."""

    def __init__(self, Ocsp_Stapling_Params_s):
        super(Ocsp_Stapling_Params, self).__init__(Ocsp_Stapling_Params_s)
        self._meta_data['exclusive_attributes'].append(
            ('proxyServerPool', 'dnsResolver'))
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:ocsp-stapling-params:ocsp-stapling-paramsstate'

    def create(self, **kwargs):
        """Create the resource on the BIG-IP®.

        Uses HTTP POST to the `collection` URI to create a resource associated
        with a new unique URI on the device.

        As proxyServerPool parameter will be required
        only if useProxyServer is set to 'enabled'
        we have to use conditional to capture this logic during create.

        """
        if kwargs['useProxyServer'] == 'enabled':
            tup_par = ('proxyServerPool', 'trustedCa', 'useProxyServer')
        else:
            tup_par = ('dnsResolver', 'trustedCa', 'useProxyServer')

        self._meta_data['required_creation_parameters'].update(tup_par)
        return self._create(**kwargs)

    def update(self, **kwargs):
        """When setting useProxyServer to enable we need to supply

            proxyServerPool value as well
        """
        if 'useProxyServer' in kwargs and kwargs['useProxyServer'] == 'enabled':
            if 'proxyServerPool' not in kwargs:
                error = 'Missing proxyServerPool parameter value.'
                raise MissingUpdateParameter(error)
        if hasattr(self, 'useProxyServer'):
            if getattr(self, 'useProxyServer') == 'enabled' and 'proxyServerPool' not in self.__dict__:
                error = 'Missing proxyServerPool parameter value.'
                raise MissingUpdateParameter(error)

        self._update(**kwargs)

        return self


class One_Connects(Collection):
    """BIG-IP® One_Connect profile collection."""
    def __init__(self, profile):
        super(One_Connects, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [One_Connect]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:one-connect:one-connectstate': One_Connect}


class One_Connect(Resource):
    """BIG-IP® One_Connect resource."""
    def __init__(self, One_Connects):
        super(One_Connect, self).__init__(One_Connects)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:one-connect:one-connectstate'


class Pcps(Collection):
    """BIG-IP® Pcp profile collection."""
    def __init__(self, profile):
        super(Pcps, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Pcp]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:pcp:pcpstate': Pcp}


class Pcp(Resource):
    """BIG-IP® Pcp resource."""
    def __init__(self, Pcps):
        super(Pcp, self).__init__(Pcps)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:pcp:pcpstate'


class Pptps(Collection):
    """BIG-IP® Pptp profile collection."""
    def __init__(self, profile):
        super(Pptps, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Pptp]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:pptp:pptpstate': Pptp}


class Pptp(Resource):
    """BIG-IP® Pptp resource."""
    def __init__(self, Pptps):
        super(Pptp, self).__init__(Pptps)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:pptp:pptpstate'


class Qoes(Collection):
    """BIG-IP® Qoe profile collection."""
    def __init__(self, profile):
        super(Qoes, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Qoe]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:qoe:qoestate': Qoe}


class Qoe(Resource):
    """BIG-IP® Qoe resource."""
    def __init__(self, Qoes):
        super(Qoe, self).__init__(Qoes)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:qoe:qoestate'


class Radius_s(Collection):
    """BIG-IP® Radius profile collection."""
    def __init__(self, profile):
        super(Radius_s, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Radius]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:radius:radiusstate': Radius}


class Radius(Resource):
    """BIG-IP® Radius resource."""
    def __init__(self, Radius_s):
        super(Radius, self).__init__(Radius_s)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:radius:radiusstate'


class Ramcaches(Collection):
    """BIG-IP® Ramcache profile collection."""
    pass


class Ramcache(Resource):
    """BIG-IP® Ramcache resource."""
    pass


class Request_Adapts(Collection):
    """BIG-IP® Request_Adapt profile collection."""
    def __init__(self, profile):
        super(Request_Adapts, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Request_Adapt]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:request-adapt:request-adaptstate': Request_Adapt}


class Request_Adapt(Resource):
    """BIG-IP® Request_Adapt resource."""
    def __init__(self, Request_Adapts):
        super(Request_Adapt, self).__init__(Request_Adapts)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:request-adapt:request-adaptstate'


class Request_Logs(Collection):
    """BIG-IP® Request_Log profile collection."""
    def __init__(self, profile):
        super(Request_Logs, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Request_Log]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:request-log:request-logstate': Request_Log}


class Request_Log(Resource):
    """BIG-IP® Request_Log resource."""
    def __init__(self, Request_Logs):
        super(Request_Log, self).__init__(Request_Logs)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:request-log:request-logstate'


class Response_Adapts(Collection):
    """BIG-IP® Response_Adapt profile collection."""
    def __init__(self, profile):
        super(Response_Adapts, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Response_Adapt]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:response-adapt:\
            response-adaptstate': Response_Adapt}


class Response_Adapt(Resource):
    """BIG-IP® Response_Adapt resource."""
    def __init__(self, Response_Adapts):
        super(Response_Adapt, self).__init__(Response_Adapts)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:response-adapt:response-adaptstate'


class Rewrites(Collection):
    """BIG-IP® Rewrite profile collection."""
    def __init__(self, profile):
        super(Rewrites, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Rewrite]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:rewrite:rewritestate': Rewrite}


class Rewrite(Resource):
    """BIG-IP® Rewrite resource."""
    def __init__(self, Rewrites):
        super(Rewrite, self).__init__(Rewrites)
        self._meta_data['allowed_lazy_attributes'] = []
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:rewrite:rewritestate'
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:rewrite:uri-rules:uri-rulescollectionstate':
             Uri_Rules_s}


class Uri_Rules_s(Collection):
    """BIG-IP® Rewrite sub-collection."""
    def __init__(self, Rewrite):
        super(Uri_Rules_s, self).__init__(Rewrite)
        self._meta_data['allowed_lazy_attributes'] = [Uri_Rules]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:rewrite:uri-rules:uri-rulesstate': Uri_Rules}


class Uri_Rules(Resource):
    """BIG-IP® URI Rules resource"""
    def __init__(self, Uri_Rules_s):
        super(Uri_Rules, self).__init__(Uri_Rules_s)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:rewrite:uri-rules:uri-rulesstate'


class Rtsps(Collection):
    """BIG-IP® Rtsp profile collection."""
    def __init__(self, profile):
        super(Rtsps, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Rtsp]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:rtsp:rtspstate': Rtsp}


class Rtsp(Resource):
    """BIG-IP® Rtsp resource."""
    def __init__(self, Rtsps):
        super(Rtsp, self).__init__(Rtsps)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:rtsp:rtspstate'


class Sctps(Collection):
    """BIG-IP® Sctp profile collection."""
    def __init__(self, profile):
        super(Sctps, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Sctp]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:sctp:sctpstate': Sctp}


class Sctp(Resource):
    """BIG-IP® Sctp resource."""
    def __init__(self, Sctps):
        super(Sctp, self).__init__(Sctps)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:sctp:sctpstate'


class Server_Ldaps(Collection):
    """BIG-IP® Server_Ldap profile collection."""
    def __init__(self, profile):
        super(Server_Ldaps, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Server_Ldap]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:server-ldap:server-ldapstate': Server_Ldap}
        self._meta_data['minimum_version'] = '11.6.0'


class Server_Ldap(Resource):
    """BIG-IP® Server_Ldap resource."""
    def __init__(self, Server_Ldaps):
        super(Server_Ldap, self).__init__(Server_Ldaps)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:server-ldap:server-ldapstate'


class Server_Ssls(Collection):
    """BIG-IP® Server_Ssl profile collection."""
    def __init__(self, profile):
        super(Server_Ssls, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Server_Ssl]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:server-ssl:server-sslstate': Server_Ssl}


class Server_Ssl(Resource):
    """BIG-IP® Server_Ssl resource."""
    def __init__(self, Server_Ssls):
        super(Server_Ssl, self).__init__(Server_Ssls)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:server-ssl:server-sslstate'


class Sips(Collection):
    """BIG-IP® Sip profile collection."""
    def __init__(self, profile):
        super(Sips, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Sip]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:sip:sipstate': Sip}


class Sip(Resource):
    """BIG-IP® Sip resource."""
    def __init__(self, Sips):
        super(Sip, self).__init__(Sips)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:sip:sipstate'


class Smtps(Collection):
    """BIG-IP® Smtp profile collection.

    .. note::
         Profile requires ASM provisioned.
    """
    def __init__(self, profile):
        super(Smtps, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Smtp]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:smtp:smtpstate': Smtp}
        self._meta_data['minimum_version'] = '11.6.0'


class Smtp(Resource):
    """BIG-IP® Smtp resource."""
    def __init__(self, Smtps):
        super(Smtp, self).__init__(Smtps)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:smtp:smtpstate'


class Smtps_s(Collection):
    """BIG-IP® Smtps profile collection."""
    def __init__(self, profile):
        super(Smtps_s, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [SmtpS]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:smtps:smtpsstate': SmtpS}


class SmtpS(Resource):
    """BIG-IP® Smtps resource."""
    def __init__(self, Smtps_s):
        super(SmtpS, self).__init__(Smtps_s)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:smtps:smtpsstate'


class Socks_s(Collection):
    """BIG-IP® Socks profile collection."""
    def __init__(self, profile):
        super(Socks_s, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Socks]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:socks:socksstate': Socks}


class Socks(Resource):
    """BIG-IP® Socks resource."""
    def __init__(self, Socks_s):
        super(Socks, self).__init__(Socks_s)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:socks:socksstate'
        self._meta_data['required_creation_parameters'].update(
            ('dnsResolver',))


class Spdys(Collection):
    """BIG-IP® Spdy profile collection."""
    def __init__(self, profile):
        super(Spdys, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Spdy]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:spdy:spdystate': Spdy}


class Spdy(Resource):
    """BIG-IP® Spdy resource."""
    def __init__(self, Spdys):
        super(Spdy, self).__init__(Spdys)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:spdy:spdystate'


class Statistics_s(Collection):
    """BIG-IP® Statistics profile collection."""
    def __init__(self, profile):
        super(Statistics_s, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Statistics]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:statistics:statisticsstate': Statistics}


class Statistics(Resource):
    """BIG-IP® Statistics resource."""
    def __init__(self, Statistics_s):
        super(Statistics, self).__init__(Statistics_s)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:statistics:statisticsstate'


class Streams(Collection):
    """BIG-IP® Stream profile collection."""
    def __init__(self, profile):
        super(Streams, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Stream]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:stream:streamstate': Stream}


class Stream(Resource):
    """BIG-IP® Stream resource."""
    def __init__(self, Streams):
        super(Stream, self).__init__(Streams)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:stream:streamstate'


class Tcps(Collection):
    """BIG-IP® Tcp profile collection."""
    def __init__(self, profile):
        super(Tcps, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Tcp]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:tcp:tcpstate': Tcp}


class Tcp(Resource):
    """BIG-IP® Tcp resource."""
    def __init__(self, Tcps):
        super(Tcp, self).__init__(Tcps)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:tcp:tcpstate'


class Tftps(Collection):
    """BIG-IP® Tftp profile collection."""
    def __init__(self, profile):
        super(Tftps, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Tftp]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:tftp:tftpstate': Tftp}
        self._meta_data['minimum_version'] = '12.0.0'


class Tftp(Resource):
    """BIG-IP® Tftp resource."""
    def __init__(self, Tftps):
        super(Tftp, self).__init__(Tftps)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:tftp:tftpstate'


class Udps(Collection):
    """BIG-IP® Udp profile collection."""
    def __init__(self, profile):
        super(Udps, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Udp]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:udp:udpstate': Udp}


class Udp(Resource):
    """BIG-IP® Udp resource."""
    def __init__(self, Udps):
        super(Udp, self).__init__(Udps)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:udp:udpstate'


class Wa_Caches(Collection):
    """BIG-IP® Wa_Cache profile collection."""
    pass


class Wa_Cache(Resource):
    """BIG-IP® Wa_Cache resource."""
    pass


class Web_Accelerations(Collection):
    """BIG-IP® Web_Acceleration profile collection."""
    def __init__(self, profile):
        super(Web_Accelerations, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Web_Acceleration]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:web-acceleration:\
            web-accelerationstate': Web_Acceleration}


class Web_Acceleration(Resource):
    """BIG-IP® Web_Acceleration resource."""
    def __init__(self, Web_Accelerations):
        super(Web_Acceleration, self).__init__(Web_Accelerations)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:web-acceleration:web-accelerationstate'


class Web_Securitys(Collection):
    """BIG-IP® Web_Security profile collection."""
    def __init__(self, profile):
        super(Web_Securitys, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Websecurity]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:websecurity:websecuritystate': Websecurity}


class Websecurity(Resource):
    """BIG-IP® Web_Security resource."""
    def __init__(self, Web_Securitys):
        super(Websecurity, self).__init__(Web_Securitys)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:web-security:web-securitystate'

    def create(self, **kwargs):
        """Create is not supported for Web Security

        :raises: UnsupportedOperation
        """
        raise UnsupportedOperation(
            "%s does not support the create method" % self.__class__.__name__
        )

    def update(self, **kwargs):
        """Update is not supported for Web Security

        :raises: UnsupportedOperation
        """
        raise UnsupportedOperation(
            "%s does not support the update method" % self.__class__.__name__
        )

    def refresh(self, **kwargs):
        """Refresh is not supported for Web Security

        :raises: UnsupportedOperation
        """
        raise UnsupportedOperation(
            "%s does not support the refresh method" % self.__class__.__name__
        )

    def delete(self, **kwargs):
        """Delete is not supported for Web Security

        :raises: UnsupportedOperation
        """
        raise UnsupportedOperation(
            "%s does not support the delete method" % self.__class__.__name__
        )


class Xmls(Collection):
    """BIG-IP® Xml profile collection."""
    def __init__(self, profile):
        super(Xmls, self).__init__(profile)
        self._meta_data['allowed_lazy_attributes'] = [Xml]
        self._meta_data['attribute_registry'] = \
            {'tm:ltm:profile:xml:xmlstate': Xml}


class Xml(Resource):
    """BIG-IP® Xml resource."""
    def __init__(self, Xmls):
        super(Xml, self).__init__(Xmls)
        self._meta_data['required_json_kind'] = \
            'tm:ltm:profile:xml:xmlstate'



# coding=utf-8
#
# Copyright 2016 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""BIG-IP Local Traffic Manager (LTM) SNAT Translation module.

REST URI
    ``https://localhost/mgmt/tm/ltm/snat-translation?ver=11.6.0``

GUI Path
    ``Local Traffic --> Address Translation --> Address Translation List``

REST Kind
    ``tm:ltm:snat-translation:*``
"""

from f5.bigip.mixins import ExclusiveAttributesMixin
from f5.bigip.resource import Collection
from f5.bigip.resource import Resource


class Snat_Translations(Collection):
    """BIG-IP® SNAT Translation collection."""
    def __init__(self, ltm):
        super(Snat_Translations, self).__init__(ltm)
        self._meta_data['allowed_lazy_attributes'] = [Snat_Translation]
        self._meta_data['attribute_registry'] = {
            'tm:ltm:snat-translation:snat-translationstate': Snat_Translation
        }


class Snat_Translation(ExclusiveAttributesMixin, Resource):
    """BIG-IP® SNAT Translation"""
    def __init__(self, Snat_Translations):
        super(Snat_Translation, self).__init__(Snat_Translations)
        self._meta_data['required_json_kind'] =\
            "tm:ltm:snat-translation:snat-translationstate"
        self._meta_data['read_only_attributes'].append('address')
        self._meta_data['exclusive_attributes'].append(('enabled', 'disabled'))



# coding=utf-8
#
# Copyright 2017 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from f5.bigip.resource import AsmResource
from f5.bigip.resource import Collection


class Websocket_Urls_s(Collection):
    """BIG-IP® ASM Websocket Urls sub-collection."""
    def __init__(self, policy):
        super(Websocket_Urls_s, self).__init__(policy)
        self._meta_data['object_has_stats'] = False
        self._meta_data['minimum_version'] = '12.1.0'
        self._meta_data['allowed_lazy_attributes'] = [Websocket_Url]
        self._meta_data['required_json_kind'] = 'tm:asm:policies:websocket-urls:websocket-urlcollectionstate'
        self._meta_data['attribute_registry'] = {
            'tm:asm:policies:websocket-urls:websocket-urlstate': Websocket_Url
        }


class Websocket_Url(AsmResource):
    """BIG-IP® ASM Websocket UrlResource."""
    def __init__(self, websocket_urls_s):
        super(Websocket_Url, self).__init__(websocket_urls_s)
        self._meta_data['required_json_kind'] = 'tm:asm:policies:websocket-urls:websocket-urlstate'
        self._meta_data['required_creation_parameters'].update(
            ('checkPayload',)
        )

    def create(self, **kwargs):
        """Custom create method to accommodate different endpoint behavior."""
        self._check_create_parameters(**kwargs)
        if kwargs['checkPayload'] is True:
            self._meta_data['minimum_additional_parameters'] = {
                'allowTextMessage', 'allowJsonMessage', 'allowBinaryMessage'
            }
        if 'allowTextMessage' in kwargs:
            self._meta_data['required_creation_parameters'].update((
                'plainTextProfileReference',)
            )
        if 'allowJsonMessage' in kwargs:
            self._meta_data['required_creation_parameters'].update((
                'jsonProfileReference',)
            )
        return super(Websocket_Url, self)._create(**kwargs)



# coding=utf-8
#
#  Copyright 2014-2016 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""iWorkflow® tenants module

REST URI
    ``http://localhost/mgmt/cm/cloud/tenants``

GUI Path
    ``Clouds and Services --> Tenants``

REST Kind
    ``cm:cloud:tenants:*``
"""

from f5.iworkflow.resource import Collection
from f5.iworkflow.resource import OrganizingCollection
from f5.iworkflow.resource import Resource


class Tenants_s(Collection):
    def __init__(self, cloud):
        super(Tenants_s, self).__init__(cloud)
        self._meta_data['required_json_kind'] = \
            'cm:cloud:tenants:tenantcollectionworkerstate'
        self._meta_data['allowed_lazy_attributes'] = [Tenant]
        self._meta_data['attribute_registry'] = {
            'cm:cloud:tenants:tenantworkerstate': Tenant
        }


class Tenant(Resource):
    def __init__(self, tenants_s):
        super(Tenant, self).__init__(tenants_s)
        self._meta_data['required_creation_parameters'] = {'name', }
        self._meta_data['required_load_parameters'] = {'name', }
        self._meta_data['required_json_kind'] = \
            'cm:cloud:tenants:tenantworkerstate'
        self._meta_data['allowed_lazy_attributes'] = [Services]
        self._meta_data['attribute_registry'] = {'': Services}


class Services(OrganizingCollection):
    def __init__(self, tenant):
        super(Services, self).__init__(tenant)
        self._meta_data['allowed_lazy_attributes'] = [Iapps]


class Iapps(Collection):
    def __init__(self, services):
        super(Iapps, self).__init__(services)
        self._meta_data['required_json_kind'] = \
            'cm:cloud:tenants:tenantservicecollectionstate'
        self._meta_data['allowed_lazy_attributes'] = [Iapp]
        self._meta_data['attribute_registry'] = {
            'cm:cloud:tenants:tenantserviceinstance': Iapp
        }


class Iapp(Resource):
    def __init__(self, iapps):
        super(Iapp, self).__init__(iapps)
        self._meta_data['required_creation_parameters'] = {'name', }
        self._meta_data['required_load_parameters'] = {'name', }
        self._meta_data['required_json_kind'] = \
            'cm:cloud:tenants:tenantserviceinstance'



# coding=utf-8
#
#  Copyright 2018 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""Directory: cm module: failover-status.

REST URI
    ``https://localhost/mgmt/tm/cm/failover-status``

GUI Path
    ``XXX``

REST Kind
    ``tm:cm:failover-status:*``
"""

from f5.bigip.resource import UnnamedResource


class Failover_Status(UnnamedResource):
    """BIG-IP® cluster resource"""
    def __init__(self, cm):
        super(Failover_Status, self).__init__(cm)
        self._meta_data['required_json_kind'] =\
            "tm:cm:failover-status:failover-statusstats"



# coding=utf-8
#
# Copyright 2016 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""BIG-IP® Network self-ip module.

.. note::

    Self IPs path does not match their kind or URI because the string ``self``
    causes problems in Python because it is a reserved word.

REST URI
    ``http://localhost/mgmt/tm/net/self``

GUI Path
    ``Network --> Self IPs``

REST Kind
    ``tm:net:self:*``
"""

from f5.bigip.resource import Collection
from f5.bigip.resource import Resource


class Selfips(Collection):
    """BIG-IP® network Self-IP collection

    .. note::

        The objects in the collection are actually called 'self' in
        iControlREST, but obviously this will cause problems in Python so we
        changed its name to Selfip.
    """
    def __init__(self, net):
        super(Selfips, self).__init__(net)
        self._meta_data['allowed_lazy_attributes'] = [Selfip]
        self._meta_data['attribute_registry'] =\
            {'tm:net:self:selfstate': Selfip}
        # Override the URI to have self instead of the constructed selfip
        self._meta_data['uri'] =\
            self._meta_data['container']._meta_data['uri'] + "self/"


class Selfip(Resource):
    '''BIG-IP® Self-IP resource

    Use this object to create, refresh, update, delete, and load self ip
    configuration on the BIG-IP®.  This requires that a
    :class:`~f5.BIG-IP.network.vlan.VLAN` object be present on the system and
    that object's :attrib:`fullPath` be used as the VLAN name.

    The address that is used for create is a *<ipaddress>/<netmask>*.  For
    example ``192.168.1.1/32``.

    .. note::

        The object is actually called ``self`` in iControlREST, but obviously
        this will cause problems in Python so we changed its name to
        ``Selfip``.
    '''
    def __init__(self, selfip_s):
        super(Selfip, self).__init__(selfip_s)
        self._meta_data['required_json_kind'] = 'tm:net:self:selfstate'
        self._meta_data['required_creation_parameters'].update(
            ('address', 'vlan'))



# coding=utf-8
#
#  Copyright 2018 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""BIG-IP® GTM monitor submodule.

REST URI
    ``http://localhost/mgmt/tm/gtm/monitor/``

GUI Path
    ``DNS --> GSLB --> Monitors``

REST Kind
    ``tm:gtm:monitor*``
"""

from f5.bigip.mixins import UpdateMonitorMixin
from f5.bigip.resource import Collection
from f5.bigip.resource import OrganizingCollection
from f5.bigip.resource import Resource


class Monitor(OrganizingCollection):
    def __init__(self, gtm):
        super(Monitor, self).__init__(gtm)
        self._meta_data['allowed_lazy_attributes'] = [
            Bigips,
            Bigip_Links,
            Externals,
            Firepass_s,
            Ftps,
            Gateway_Icmps,
            Gtps,
            Https,
            Https_s,
            Imaps,
            Ldaps,
            Mssqls,
            Mysqls,
            Nntps,
            Oracles,
            Pop3s,
            Postgresqls,
            Radius_s,
            Radius_Accountings,
            Real_Servers,
            Scripteds,
            Sips,
            Smtps,
            Snmps,
            Snmp_Links,
            Soaps,
            Tcps,
            Tcp_Half_Opens,
            Udps,
            Waps,
            Wmis
        ]


class Bigips(Collection):
    def __init__(self, monitor):
        super(Bigips, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Bigip]
        self._meta_data['attribute_registry'] = {
            'tm:gtm:monitor:bigip:bigipstate': Bigip
        }


class Bigip(UpdateMonitorMixin, Resource):
    def __init__(self, bigips):
        super(Bigip, self).__init__(bigips)
        self._meta_data['required_json_kind'] = 'tm:gtm:monitor:bigip:bigipstate'


class Bigip_Links(Collection):
    def __init__(self, monitor):
        super(Bigip_Links, self).__init__(monitor)
        fixed = self._meta_data['uri'].replace('_', '-')
        self._meta_data['uri'] = fixed
        self._meta_data['allowed_lazy_attributes'] = [Bigip_Link]
        self._meta_data['attribute_registry'] = {
            'tm:gtm:monitor:bigip-link:bigip-linkstate': Bigip_Link
        }


class Bigip_Link(UpdateMonitorMixin, Resource):
    def __init__(self, bigip_links):
        super(Bigip_Link, self).__init__(bigip_links)
        self._meta_data['required_json_kind'] = 'tm:gtm:monitor:bigip-link:bigip-linkstate'


class Externals(Collection):
    def __init__(self, monitor):
        super(Externals, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [External]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:external:externalstate': External}


class External(UpdateMonitorMixin, Resource):
    def __init__(self, externals):
        super(External, self).__init__(externals)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:external:externalstate'


class Firepass_s(Collection):
    def __init__(self, monitor):
        super(Firepass_s, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Firepass]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:firepass:firepassstate': Firepass}


class Firepass(UpdateMonitorMixin, Resource):
    def __init__(self, firepass_s):
        super(Firepass, self).__init__(firepass_s)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:firepass:firepassstate'


class Ftps(Collection):
    def __init__(self, monitor):
        super(Ftps, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Ftp]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:ftp:ftpstate': Ftp}


class Ftp(UpdateMonitorMixin, Resource):
    def __init__(self, ftps):
        super(Ftp, self).__init__(ftps)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:ftp:ftpstate'


class Gateway_Icmps(Collection):
    def __init__(self, monitor):
        super(Gateway_Icmps, self).__init__(monitor)
        fixed = self._meta_data['uri'].replace('_', '-')
        self._meta_data['uri'] = fixed
        self._meta_data['allowed_lazy_attributes'] = [Gateway_Icmp]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:gateway-icmp:gateway-icmpstate': Gateway_Icmp}


class Gateway_Icmp(UpdateMonitorMixin, Resource):
    def __init__(self, gateway_icmps):
        super(Gateway_Icmp, self).__init__(gateway_icmps)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:gateway-icmp:gateway-icmpstate'


class Gtps(Collection):
    def __init__(self, monitor):
        super(Gtps, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Gtp]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:gtp:gtpstate': Gtp}


class Gtp(UpdateMonitorMixin, Resource):
    def __init__(self, gtps):
        super(Gtp, self).__init__(gtps)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:gtp:gtpstate'


class Https(Collection):
    def __init__(self, monitor):
        super(Https, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Http]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:http:httpstate': Http}


class Http(UpdateMonitorMixin, Resource):
    def __init__(self, https):
        super(Http, self).__init__(https)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:http:httpstate'


class Https_s(Collection):
    """BIG-IP® Https monitor collection."""
    def __init__(self, monitor):
        super(Https_s, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [HttpS]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:https:httpsstate': HttpS}


class HttpS(UpdateMonitorMixin, Resource):
    """BIG-IP® Https monitor resource."""
    def __init__(self, https_s):
        super(HttpS, self).__init__(https_s)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:https:httpsstate'


class Imaps(Collection):
    """BIG-IP® Imap monitor collection."""
    def __init__(self, monitor):
        super(Imaps, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Imap]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:imap:imapstate': Imap}


class Imap(UpdateMonitorMixin, Resource):
    """BIG-IP® Imap monitor resource."""
    def __init__(self, imaps):
        super(Imap, self).__init__(imaps)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:imap:imapstate'


class Ldaps(Collection):
    """BIG-IP® Ldap monitor collection."""
    def __init__(self, monitor):
        super(Ldaps, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Ldap]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:ldap:ldapstate': Ldap}


class Ldap(UpdateMonitorMixin, Resource):
    """BIG-IP® Ldap monitor resource."""
    def __init__(self, ldaps):
        super(Ldap, self).__init__(ldaps)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:ldap:ldapstate'


class Mssqls(Collection):
    """BIG-IP® Mssql monitor collection."""
    def __init__(self, monitor):
        super(Mssqls, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Mssql]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:mssql:mssqlstate': Mssql}


class Mssql(UpdateMonitorMixin, Resource):
    """BIG-IP® Mssql monitor resource."""
    def __init__(self, mssqls):
        super(Mssql, self).__init__(mssqls)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:mssql:mssqlstate'


class Mysqls(Collection):
    """BIG-IP® MySQL monitor collection."""
    def __init__(self, monitor):
        super(Mysqls, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Mysql]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:mysql:mysqlstate': Mysql}


class Mysql(UpdateMonitorMixin, Resource):
    """BIG-IP® MySQL monitor resource."""
    def __init__(self, mysqls):
        super(Mysql, self).__init__(mysqls)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:mysql:mysqlstate'


class Nntps(Collection):
    """BIG-IP® Nntps monitor collection."""
    def __init__(self, monitor):
        super(Nntps, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Nntp]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:nntp:nntpstate': Nntp}


class Nntp(UpdateMonitorMixin, Resource):
    """BIG-IP® Nntps monitor resource."""
    def __init__(self, nntps):
        super(Nntp, self).__init__(nntps)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:nntp:nntpstate'


class Nones(Collection):
    """BIG-IP® None monitor collection."""
    def __init__(self, monitor):
        super(Nones, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [NONE]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:none:nonestate': NONE}


class NONE(UpdateMonitorMixin, Resource):
    """BIG-IP® None monitor resource."""
    def __init__(self, nones):
        super(NONE, self).__init__(nones)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:none:nonestate'


class Oracles(Collection):
    """BIG-IP® Oracle monitor collection."""
    def __init__(self, monitor):
        super(Oracles, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Oracle]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:oracle:oraclestate': Oracle}


class Oracle(UpdateMonitorMixin, Resource):
    """BIG-IP® Oracle monitor resource."""
    def __init__(self, oracles):
        super(Oracle, self).__init__(oracles)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:oracle:oraclestate'


class Pop3s(Collection):
    """BIG-IP® Pop3 monitor collection."""
    def __init__(self, monitor):
        super(Pop3s, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Pop3]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:pop3:pop3state': Pop3}


class Pop3(UpdateMonitorMixin, Resource):
    """BIG-IP® Pop3 monitor resource."""
    def __init__(self, pop3s):
        super(Pop3, self).__init__(pop3s)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:pop3:pop3state'


class Postgresqls(Collection):
    """BIG-IP® PostGRES SQL monitor collection."""
    def __init__(self, monitor):
        super(Postgresqls, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Postgresql]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:postgresql:postgresqlstate': Postgresql}


class Postgresql(UpdateMonitorMixin, Resource):
    """BIG-IP® PostGRES SQL monitor resource."""
    def __init__(self, postgresqls):
        super(Postgresql, self).__init__(postgresqls)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:postgresql:postgresqlstate'


class Radius_s(Collection):
    """BIG-IP® radius monitor collection."""
    def __init__(self, monitor):
        super(Radius_s, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Radius]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:radius:radiusstate': Radius}


class Radius(UpdateMonitorMixin, Resource):
    """BIG-IP® radius monitor resource."""
    def __init__(self, radius_s):
        super(Radius, self).__init__(radius_s)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:radius:radiusstate'


class Radius_Accountings(Collection):
    """BIG-IP® radius accounting monitor collection."""
    def __init__(self, monitor):
        super(Radius_Accountings, self).__init__(monitor)
        fixed = self._meta_data['uri'].replace('_', '-')
        self._meta_data['uri'] = fixed
        self._meta_data['allowed_lazy_attributes'] = [Radius_Accounting]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:radius-accounting:radius-accountingstate':
             Radius_Accounting}


class Radius_Accounting(UpdateMonitorMixin, Resource):
    """BIG-IP® radius accounting monitor resource."""
    def __init__(self, radius_accountings):
        super(Radius_Accounting, self).__init__(radius_accountings)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:radius-accounting:radius-accountingstate'


class Real_Servers(Collection):
    """BIG-IP® real-server monitor collection."""
    def __init__(self, monitor):
        super(Real_Servers, self).__init__(monitor)
        fixed = self._meta_data['uri'].replace('_', '-')
        self._meta_data['uri'] = fixed
        self._meta_data['allowed_lazy_attributes'] = [Real_Server]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:real-server:real-serverstate': Real_Server}


class Real_Server(UpdateMonitorMixin, Resource):
    """BIG-IP® real-server monitor resource."""
    def __init__(self, real_servers):
        super(Real_Server, self).__init__(real_servers)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:real-server:real-serverstate'

    def update(self, **kwargs):
        """Change the configuration of the resource on the device.

        This method uses Http PUT alter the service state on the device.

        The attributes of the instance will be packaged as a dictionary.  That
        dictionary will be updated with kwargs.  It is then submitted as JSON
        to the device.  Various edge cases are handled:

        * read-only attributes that are unchangeable are removed
        * ``tmCommand`` attribute removed prior to PUT
        * ``agent`` attribute removed prior to PUT
        * ``post`` attribute removed prior to PUT


        :param kwargs: keys and associated values to alter on the device

        """
        self.__dict__.pop('tmCommand', '')
        self.__dict__.pop('agent', '')
        self.__dict__.pop('method', '')
        super(Real_Server, self).update(**kwargs)


class Scripteds(Collection):
    """BIG-IP® scripted monitor collection."""
    def __init__(self, monitor):
        super(Scripteds, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Scripted]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:scripted:scriptedstate': Scripted}


class Scripted(UpdateMonitorMixin, Resource):
    """BIG-IP® scripted monitor resource."""
    def __init__(self, scripteds):
        super(Scripted, self).__init__(scripteds)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:scripted:scriptedstate'


class Sips(Collection):
    """BIG-IP® Sip monitor collection."""
    def __init__(self, monitor):
        super(Sips, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Sip]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:sip:sipstate': Sip}


class Sip(UpdateMonitorMixin, Resource):
    """BIG-IP® Sip monitor resource."""
    def __init__(self, sips):
        super(Sip, self).__init__(sips)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:sip:sipstate'


class Smtps(Collection):
    """BIG-IP® Smtp monitor collection."""
    def __init__(self, monitor):
        super(Smtps, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Smtp]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:smtp:smtpstate': Smtp}


class Smtp(UpdateMonitorMixin, Resource):
    """BIG-IP® Smtp monitor resource."""
    def __init__(self, smtps):
        super(Smtp, self).__init__(smtps)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:smtp:smtpstate'


class Snmps(Collection):
    """BIG-IP® Smtp monitor collection."""
    def __init__(self, monitor):
        super(Snmps, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Snmp]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:snmp:snmpstate': Snmp}


class Snmp(UpdateMonitorMixin, Resource):
    """BIG-IP® Smtp monitor resource."""
    def __init__(self, snmps):
        super(Snmp, self).__init__(snmps)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:snmp:snmpstate'


class Snmp_Links(Collection):
    def __init__(self, monitor):
        super(Snmp_Links, self).__init__(monitor)
        fixed = self._meta_data['uri'].replace('_', '-')
        self._meta_data['uri'] = fixed
        self._meta_data['allowed_lazy_attributes'] = [Snmp_Link]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:snmp-link:snmp-linkstate': Snmp_Link}


class Snmp_Link(UpdateMonitorMixin, Resource):
    def __init__(self, snmp_links):
        super(Snmp_Link, self).__init__(snmp_links)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:snmp-link:snmp-linkstate'


class Soaps(Collection):
    def __init__(self, monitor):
        super(Soaps, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Soap]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:soap:soapstate': Soap}


class Soap(UpdateMonitorMixin, Resource):
    def __init__(self, soaps):
        super(Soap, self).__init__(soaps)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:soap:soapstate'


class Tcps(Collection):
    def __init__(self, monitor):
        super(Tcps, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Tcp]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:tcp:tcpstate': Tcp}


class Tcp(UpdateMonitorMixin, Resource):
    def __init__(self, tcps):
        super(Tcp, self).__init__(tcps)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:tcp:tcpstate'


class Tcp_Half_Opens(Collection):
    def __init__(self, monitor):
        super(Tcp_Half_Opens, self).__init__(monitor)
        fixed = self._meta_data['uri'].replace('_', '-')
        self._meta_data['uri'] = fixed
        self._meta_data['allowed_lazy_attributes'] = [Tcp_Half_Open]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:tcp-half-open:tcp-half-openstate': Tcp_Half_Open}


class Tcp_Half_Open(UpdateMonitorMixin, Resource):
    def __init__(self, tcp_half_opens):
        super(Tcp_Half_Open, self).__init__(tcp_half_opens)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:tcp-half-open:tcp-half-openstate'


class Udps(Collection):
    def __init__(self, monitor):
        super(Udps, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Udp]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:udp:udpstate': Udp}


class Udp(UpdateMonitorMixin, Resource):
    def __init__(self, udps):
        super(Udp, self).__init__(udps)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:udp:udpstate'


class Waps(Collection):
    def __init__(self, monitor):
        super(Waps, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Wap]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:wap:wapstate': Wap}


class Wap(UpdateMonitorMixin, Resource):
    def __init__(self, waps):
        super(Wap, self).__init__(waps)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:wap:wapstate'


class Wmis(Collection):
    def __init__(self, monitor):
        super(Wmis, self).__init__(monitor)
        self._meta_data['allowed_lazy_attributes'] = [Wmi]
        self._meta_data['attribute_registry'] =\
            {'tm:gtm:monitor:wmi:wmistate': Wmi}


class Wmi(UpdateMonitorMixin, Resource):
    def __init__(self, wmis):
        super(Wmi, self).__init__(wmis)
        self._meta_data['required_json_kind'] =\
            'tm:gtm:monitor:wmi:wmistate'
        self._meta_data['read_only_attributes'] =\
            ['agent', 'post', 'method']

    def update(self, **kwargs):
        """Change the configuration of the resource on the device.

        This method uses Http PUT alter the service state on the device.

        The attributes of the instance will be packaged as a dictionary.  That
        dictionary will be updated with kwargs.  It is then submitted as JSON
        to the device.  Various edge cases are handled:

        * read-only attributes that are unchangeable are removed
        * ``agent`` attribute removed prior to PUT
        * ``post`` attribute removed prior to PUT
        * ``method`` attribute removed prior to PUT

        :param kwargs: keys and associated values to alter on the device

        """
        self.__dict__.pop('agent', '')
        self.__dict__.pop('post', '')
        self.__dict__.pop('method', '')
        super(Wmi, self).update(**kwargs)



# coding=utf-8
#
# Copyright 2016 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""BIG-IP® Shared (shared) module

REST URI
    ``http://localhost/mgmt/tm/shared/``

GUI Path
    ``System``

REST Kind
    N/A -- HTTP GET returns an error
"""

from f5.bigip.resource import PathElement
from f5.bigip.tm.shared.bigip_failover_state import Bigip_Failover_State
from f5.bigip.tm.shared.licensing import Licensing


class Shared(PathElement):
    """BIG-IP® System Shared Utility (shared) collection."""
    def __init__(self, bigip):
        super(Shared, self).__init__(bigip)
        self._meta_data['allowed_lazy_attributes'] = [
            Licensing,
            Bigip_Failover_State,
        ]



# coding=utf-8
#
#  Copyright 2014-2017 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""BIG-IP® Global Traffic Manager™ (GTM®) Topology Records module.

REST URI
    ``http://localhost/mgmt/tm/gtm/topology``

GUI Path
    ``DNS --> GSLB : Topology : Records``

REST Kind
    ``tm:gtm:topology:*``
"""
from distutils.version import LooseVersion
from f5.bigip.resource import Collection
from f5.bigip.resource import Resource
from f5.sdk_exception import InvalidName
from f5.sdk_exception import UnsupportedOperation
from f5.sdk_exception import UnsupportedTmosVersion
from f5.sdk_exception import URICreationCollision


from requests import HTTPError


class Topology_s(Collection):
    """BIG-IP® GTM Topology collection

    Due to an bug where attempts to create/modify/load/delete etc. of this
    resource in 12.0.0 Final, we have disabled this resource for version
    12.0.0
    """
    def __init__(self, gtm):
        super(Topology_s, self).__init__(gtm)
        self._meta_data['allowed_lazy_attributes'] = [Topology]
        self._meta_data['attribute_registry'] = \
            {'tm:gtm:topology:topologystate': Topology}
        self.disable_resource_for_version('12.0.0')

    def disable_resource_for_version(self, version):
        tmos_v = self._meta_data['bigip'].tmos_version
        if LooseVersion(tmos_v) == LooseVersion(version):
            error = "There was an attempt to access resource: \n{}\n which " \
                    "is disabled for the device's TMOS version: {}.".\
                format(self._meta_data['uri'], tmos_v)
            raise UnsupportedTmosVersion(error)


class Topology(Resource):
    """BIG-IP® GTM Topology resource"""

    def __init__(self, topology_s):
        super(Topology, self).__init__(topology_s)
        self._meta_data['required_json_kind'] = \
            'tm:gtm:topology:topologystate'

    def _create(self, **kwargs):
        """This method needs to be created due to few bugs in 11.5.4 and 11.6.1


        The issue arises when you attempt to create a Topology Record resource
        and while the operation succeeds BIGIP responds with 404.
        """
        if 'uri' in self._meta_data:
            error = "There was an attempt to assign a new uri to this " \
                    "resource, the _meta_data['uri'] is %s and it should" \
                    " not be changed." % (self._meta_data['uri'])
            raise URICreationCollision(error)
        self._check_exclusive_parameters(**kwargs)
        requests_params = self._handle_requests_params(kwargs)
        self._check_create_parameters(**kwargs)

        # Reduce boolean pairs as specified by the meta_data entry below
        for key1, key2 in self._meta_data['reduction_forcing_pairs']:
            kwargs = self._reduce_boolean_pair(kwargs, key1, key2)

        # Make convenience variable with short names for this method.
        _create_uri = self._meta_data['container']._meta_data['uri']
        session = self._meta_data['bigip']._meta_data['icr_session']

        # This is a bit hacky but we need to do this so we are able to
        # create a resource inside SDK properly. We also include the
        # scenario where 20x range response occurs just in case this gets
        # fixed in later release.

        try:
            response = session.post(
                _create_uri, json=kwargs, **requests_params)

        except HTTPError as err:
            if err.response.status_code != 404:
                raise
            if err.response.status_code == 404:
                kwargs['uri_as_parts'] = True
                response = session.get(_create_uri, **kwargs)

        # Make new instance of self
        return self._produce_instance(response)

    def create(self, **kwargs):
        """Custom method to implement checking of kwarg['name'] contents.

        ::Warning:
            Be sure to format the gtm topology OID string using the
            following rules:

        1) Use only a single space between each item in the topology string.
        2) Use a fully-pathed name for datacenter, isp, region, and pool
        objects.

        For example:
        "ldns: subnet 11.11.11.0/24 server: datacenter /Common/DC"


        """
        if 'name' in kwargs:
            if 'ldns' not in kwargs['name'] or 'server' not in kwargs['name']:
                raise InvalidName("Topology record name should contain both "
                                  "'ldns', 'server' keywords with their "
                                  "proper arguments")
        return self._create(**kwargs)

    def load(self, **kwargs):
        """Custom method to implement checking of kwarg['name'] contents."""
        if 'name' in kwargs:
            if 'ldns' not in kwargs['name'] or 'server' not in kwargs['name']:
                raise InvalidName("Topology record name should contain both "
                                  "'ldns', 'server' keywords with their "
                                  "proper arguments")
        kwargs['transform_name'] = True

        return self._load(**kwargs)

    def exists(self, **kwargs):
        """Providing a partition is not necessary on topology; causes errors"""
        kwargs.pop('partition', None)
        kwargs['transform_name'] = True
        return self._exists(**kwargs)

    def refresh(self, **kwargs):
        """Refresh is not supported for Topology

        :raises: UnsupportedOperation
        """
        raise UnsupportedOperation(
            "%s does not support the refresh method" %
            self.__class__.__name__
        )

    def modify(self, **patch):
        """Modify is not supported for Topology

        :raises: UnsupportedOperation
        """
        raise UnsupportedOperation(
            "%s does not support the modify method" %
            self.__class__.__name__
        )

    def update(self, **kwargs):
        """Update is not supported for Topology

        :raises: UnsupportedOperation
        """
        raise UnsupportedOperation(
            "%s does not support the update method" %
            self.__class__.__name__
        )



# coding=utf-8
#
# Copyright 2015-2016 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""BIG-IP® Global Traffic Manager™ (GTM®) module.

REST URI
    ``http://localhost/mgmt/tm/gtm/``

GUI Path
    ``DNS``

REST Kind
    ``tm:gtm:*``
"""


from f5.bigip.resource import OrganizingCollection
from f5.bigip.tm.gtm.datacenter import Datacenters
from f5.bigip.tm.gtm.global_settings import Global_Settings
from f5.bigip.tm.gtm.listener import Listeners
from f5.bigip.tm.gtm.monitor import Monitor
from f5.bigip.tm.gtm.pool import Pools
from f5.bigip.tm.gtm.region import Regions
from f5.bigip.tm.gtm.rule import Rules
from f5.bigip.tm.gtm.server import Servers
from f5.bigip.tm.gtm.topology import Topology_s
from f5.bigip.tm.gtm.wideip import Wideips


class Gtm(OrganizingCollection):
    """BIG-IP® Global Traffic Manager (GTM) organizing collection."""
    def __init__(self, tm):
        super(Gtm, self).__init__(tm)
        self._meta_data['allowed_lazy_attributes'] = [
            Datacenters,
            Global_Settings,
            Listeners,
            Monitor,
            Pools,
            Regions,
            Rules,
            Servers,
            Topology_s,
            Wideips,
        ]



# coding=utf-8
#
# Copyright 2017 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from f5.bigip.resource import AsmResource
from f5.bigip.resource import Collection
from f5.sdk_exception import UnsupportedOperation


class Character_Sets_s(Collection):
    """BIG-IP® ASM Character Sets sub-collection."""
    def __init__(self, policy):
        super(Character_Sets_s, self).__init__(policy)
        self._meta_data['object_has_stats'] = False
        self._meta_data['minimum_version'] = '11.6.0'
        self._meta_data['allowed_lazy_attributes'] = [Character_Sets]
        self._meta_data['required_json_kind'] = 'tm:asm:policies:character-sets:character-setcollectionstate'
        self._meta_data['attribute_registry'] = {
            'tm:asm:policies:character-sets:character-setstate': Character_Sets
        }


class Character_Sets(AsmResource):
    """BIG-IP® ASM Character Sets Resource."""
    def __init__(self, character_sets_s):
        super(Character_Sets, self).__init__(character_sets_s)
        self._meta_data['required_json_kind'] = 'tm:asm:policies:character-sets:character-setstate'

    def create(self, **kwargs):
        """Modify is not supported for Character Sets resource

        :raises: UnsupportedOperation
        """
        raise UnsupportedOperation(
            "%s does not support the create method" % self.__class__.__name__
        )

    def delete(self, **kwargs):
        """Delete is not supported for Character Sets resource

        :raises: UnsupportedOperation
        """
        raise UnsupportedOperation(
            "%s does not support the delete method" % self.__class__.__name__
        )



# coding=utf-8
#
#  Copyright 2014-2016 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""iWorkflow® License pool module.

REST URI
    ``http://localhost/mgmt/cm/shared/licensing/pools``

REST Kind
    ``cm:shared:licensing:pools:*``
"""

from f5.iworkflow.resource import Collection
from f5.iworkflow.resource import Resource
from f5.sdk_exception import RequiredOneOf
from six import iterkeys


class Pools_s(Collection):
    """iWorkflow® License pool collection"""
    def __init__(self, licensing):
        super(Pools_s, self).__init__(licensing)
        self._meta_data['required_json_kind'] = \
            'cm:shared:licensing:pools:licensepoolworkercollectionstate'
        self._meta_data['allowed_lazy_attributes'] = [Pool]
        self._meta_data['attribute_registry'] = {
            'cm:shared:licensing:pools:licensepoolworkerstate': Pool
        }


class Pool(Resource):
    """iWorkflow® License pool resource"""
    def __init__(self, pool_s):
        super(Pool, self).__init__(pool_s)
        self._meta_data['required_creation_parameters'] = {'baseRegKey', }
        self._meta_data['required_json_kind'] = \
            'cm:shared:licensing:pools:licensepoolworkerstate'
        self._meta_data['allowed_lazy_attributes'] = [
            Members_s
        ]
        self._meta_data['attribute_registry'] = {
            'cm:shared:licensing:pools:licensepoolmembercollectionstate':
                Members_s
        }


class Members_s(Collection):
    def __init__(self, pool):
        super(Members_s, self).__init__(pool)
        self._meta_data['required_json_kind'] = \
            'cm:shared:licensing:pools:licensepoolmembercollectionstate'
        self._meta_data['allowed_lazy_attributes'] = [Member]
        self._meta_data['attribute_registry'] = {
            'cm:shared:licensing:pools:licensepoolmemberstate': Member
        }


class Member(Resource):
    def __init__(self, members_s):
        super(Member, self).__init__(members_s)
        self._meta_data['required_json_kind'] = \
            'cm:shared:licensing:pools:licensepoolmemberstate'

        # This set is empty because the creation checking is done as
        # a required_one_of in the create() method
        self._meta_data['required_creation_parameters'] = set()

    def create(self, **kwargs):
        required_one_of = [
            ['address', 'username', 'password'],
            ['deviceReference']
        ]

        args = set(iterkeys(kwargs))

        # Check to see if the user supplied any of the sets of required
        # arguments.
        has_any = [set(x).issubset(args) for x in required_one_of]

        if len([x for x in has_any if x is True]) == 1:
            # Only one of the required argument sets can be provided
            # so if more than that are provided, it is an error.
            return self._create(**kwargs)

        raise RequiredOneOf(required_one_of)



# coding=utf-8
#
# Copyright 2017 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""BIG-IP® auth module

REST URI
    ``http://localhost/mgmt/tm/auth/tacacs``

GUI Path
    ``System --> Users --> Authentication``

REST Kind
    ``tm:auth:tacacs:*``
"""

from f5.bigip.resource import Collection
from f5.bigip.resource import Resource
from f5.sdk_exception import InvalidName


class Tacacs_s(Collection):
    """BIG-IP® tacacs server collection"""
    def __init__(self, auth):
        super(Tacacs_s, self).__init__(auth)
        self._meta_data['allowed_lazy_attributes'] = [Tacacs]
        self._meta_data['attribute_registry'] = \
            {'tm:auth:tacacs:tacacsstate': Tacacs}


class Tacacs(Resource):
    """BIG-IP® tacacs server resource"""
    def __init__(self, tacacs_s):
        super(Tacacs, self).__init__(tacacs_s)
        self._meta_data['required_json_kind'] = 'tm:auth:tacacs:tacacsstate'

    def create(self, **kwargs):
        """name has to be system-auth, raise error if not named correctly"""
        if 'name' in kwargs:
            if kwargs['name'] != 'system-auth':
                raise InvalidName("Name must be 'system-auth'")
        return self._create(**kwargs)



# Copyright 2016 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import print_function

import io
import jinja2
import json
import os
import re

from pprint import pprint as pp
from six import iterkeys


class UnexpectedOCItem(Exception):
    pass


class Kindless(Exception):
    pass


class TemplateEngine(object):
    '''Instantiate this to handle generating Python from JSON

    Currently supports generation of OrganizingCollection, and
    Collection, strings.  Also produces most of Resource classes.
    '''
    def __init__(self, template_dir, config_dir):
        self.config_dir = config_dir
        self.OC_pattern =\
            r'tm:(?P<OrgColl>\w+):(?P=OrgColl)collectionstate'
        # Initialize templates
        self.templates = {}
        self._consume_directory(template_dir, self._template_consumer)
        self.import_template = self.templates['imports']

        # Initialize configs
        self.raw_configs = {}
        self.formatted_configs = {}
        self._consume_directory(config_dir, self._config_consumer)

    def _template_consumer(self, STUB, dirname, fnames):
        for fname in fnames:
            if fname.endswith('.tmpl'):
                with io.open(os.path.join(dirname, fname)) as fh:
                    templatekey = fname[:-len('.tmpl')]
                    self.templates[templatekey] = jinja2.Template(fh.read())

    def _config_consumer(self, STUB, dirname, fnames):
        for fname in fnames:
            if fname.endswith('.json'):
                with io.open(os.path.join(dirname, fname)) as fh:
                    configkey = fname[:-len('.json')]
                    self.raw_configs[configkey] = json.load(fh)

    def _consume_directory(self, dirname, consumer):
        os.path.walk(dirname, consumer, self)

    def list_templates(self):
        pp(list(iterkeys(self.templates)))

    def list_raw_configs(self):
        pp(list(iterkeys(self.raw_configs)))

    def process_config_from_fname(self, fname):
        if not fname.endswith('.json'):
            raise Exception
        config_name = fname[:-len('.json')]
        try:
            raw_conf = self.raw_configs[config_name]
        except KeyError as ex:
            error_message = "Expected file named %r.json to exist in"\
                            " directory %r" % (config_name, self.config_dir)
            print(error_message)
            raise ex
        if 'kind' in raw_conf:
            python_as_string = self._process_config_with_kind(raw_conf)
        else:
            raise Kindless(raw_conf)
        return python_as_string, raw_conf['selfLink']

    def _handle_dashes_dots_capitals(self, raw_klass):
        print('raw_klass: %r' % raw_klass)
        dashless_klass = raw_klass.replace('-', '_')
        dotless_klass = dashless_klass.replace('.', '_')
        return '_'.join([x.capitalize() for x in dotless_klass.split('_')])

    def _build_orgcoll_import_dicts(self, raw_conf, klass):
        imports = []
        selfLinkstart = raw_conf[u"selfLink"].partition("?")[0]
        items = raw_conf[u"items"]
        for item in items:
            if item[u"reference"][u"link"].startswith(selfLinkstart):
                tempuri = item[u"reference"][u"link"]
                pre_questionmark = tempuri.partition("?")[0]
                post_selfLink = pre_questionmark[len(selfLinkstart)+1:]
                KlassName = self._handle_dashes_dots_capitals(post_selfLink)
                print(KlassName)
                imports.append({'OC': '.'+klass.lower(),
                                'module': '.'+KlassName.lower(),
                                'klass': KlassName})
            else:
                raise UnexpectedOCItem(item)
        return imports

    def _build_module_docstring(self, kind, raw_conf):
        kind_split = kind.split(':')
        conf_dict = {'ORG_COL_DIR': kind_split[1],
                     'COL_MODULE': kind_split[2],
                     'REST_URI': raw_conf['selfLink'],
                     'GUI_PATH': 'XXX',
                     'KIND_PREFIX': ':'.join(kind_split[:3])}
        mod_doc_string = self.templates['module_docstring'].render(**conf_dict)
        return mod_doc_string

    def _format_org_collection(self, org_match, kind, raw_conf):
        KlassName =\
            self._handle_dashes_dots_capitals(org_match.group('OrgColl'))
        OC_template = self.templates['OrganizingCollection']
        import_dicts = self._build_orgcoll_import_dicts(raw_conf, KlassName)
        config_dict = {'klass': KlassName,
                       'kind': kind,
                       'import_dicts': import_dicts}
        OrgCollstr = OC_template.render(**config_dict)
        imports_as_str = self.import_template.render(import_dicts=import_dicts)
        imps_as_list = imports_as_str.splitlines()
        imps_as_list.append(
            'from f5.bigip.resource import OrganizingCollection')
        imps_as_list.sort()
        imports = '\n'.join(imps_as_list)
        mdoc_string = self._build_module_docstring(kind, raw_conf)
        python_as_string =\
            mdoc_string + 3*os.linesep + imports + 3*os.linesep +\
            OrgCollstr + os.linesep
        self.formatted_configs = {KlassName: {'Python_str': python_as_string,
                                              'config_dict': config_dict,
                                              'import_dicts': import_dicts}}
        return python_as_string

    def _build_CollectionName_from_KlassName(self, KlassName):
        if KlassName.endswith('s'):
            CollectionName = KlassName+'_s'
        else:
            CollectionName = KlassName+'s'
        return CollectionName

    def _format_resource(self, kind, raw_conf):
        raw_string = kind.split(':')[-1][:-len('state')]
        assert raw_string == kind.split(':')[-2]
        KlassName = self._handle_dashes_dots_capitals(raw_string)
        container = self._build_CollectionName_from_KlassName(KlassName)
        template = self.templates['Resource']
        Resourcestr = template.render(container=container,
                                      klass=KlassName,
                                      kind=kind,
                                      attr_reg_dict={})
        import_str = 'from f5.bigip.resource import Resource'
        mdoc_string = self._build_module_docstring(kind, raw_conf)
        python_as_string =\
            mdoc_string + 3*os.linesep + import_str + 3*os.linesep +\
            Resourcestr + os.linesep
        return python_as_string

    def _format_collection(self, kind, raw_conf):
        raw_string = kind.split(':')[-1][:-len('collectionstate')]
        KlassName = self._handle_dashes_dots_capitals(raw_string)
        CollKlassName = self._build_CollectionName_from_KlassName(KlassName)
        member_klass = KlassName
        collection_template = self.templates['Collection']
        memberkind = kind.replace('collection', '')
        config_dict = {'coll_klass': CollKlassName,
                       'member_klass': member_klass,
                       'collection_container': kind.split(':')[-3],
                       'collection_attr_reg_dict': {memberkind: member_klass}}
        import_str = 'from f5.bigip.resource import Collection'
        Collstr = collection_template.render(**config_dict)
        mdoc_string = self._build_module_docstring(kind, raw_conf)
        python_as_string =\
            mdoc_string + 3*os.linesep + import_str + 3*os.linesep +\
            Collstr + os.linesep
        return python_as_string

    def _format_stats(self, kind, raw_conf):
        klass = kind.split(':')[-1][:-len('stats')]
        print(klass)
        template = self.templates['Stats']
        print(template)
        # XXX
        # return python_as_string

    def _process_config_with_kind(self, raw_conf):
        '''Use this to decide which format is called for by the kind.

        NOTE, order matters since subsequent conditions are not evaluated is an
        earlier condition is met.  In the some cases, e.g. a kind contains both
        "stats" and "state", this will become an issue.
        '''
        kind = raw_conf[u"kind"]
        org_match = re.match(self.OC_pattern, kind)
        if org_match:
            return self._format_org_collection(org_match, kind, raw_conf)
        elif 'collectionstate' in kind:
            return self._format_collection(kind, raw_conf)
        elif kind.endswith('stats'):
            return self._format_stats(kind, raw_conf)
        elif kind.endswith('state'):
            return self._format_resource(kind, raw_conf)



# coding=utf-8
#
# Copyright 2016 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""BIG-IP® system global-settings module

REST URI
    ``http://localhost/mgmt/tm/sys/global-settings``

GUI Path
    ``System --> Configuration --> Device``

REST Kind
    ``tm:sys:global-settings:*``
"""

from f5.bigip.resource import UnnamedResource


class Global_Settings(UnnamedResource):
    """BIG-IP® system global-settings resource

    The global_settings object only supports load and update because it is an
    unnamed resource.

    .. note::

        This is an unnamed resource so it has not ~Partition~Name pattern
        at the end of its URI.
    """
    def __init__(self, sys):
        super(Global_Settings, self).__init__(sys)
        self._meta_data['required_load_parameters'] = set()
        self._meta_data['required_json_kind'] =\
            'tm:sys:global-settings:global-settingsstate'



#!/usr/bin/env python
# Copyright 2017 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#

"""
This is an exceptions module holding exception classes that are of use to the
rest of the pkg building architecture.
"""

import errno

from inspect import currentframe as cf
from inspect import getframeinfo as gfi
from inspect import getouterframes as gof


class BuildError(Exception):
    """Creates a BuildError exception-type class

An excpetion class that will drop concise error messages into the terminal
rather than fill up the terminal with stacktraces.  Full stack traces can then
be generated by the person troubleshooting the issue.
    """
    default_msg = "An unknown error has occurred"
    default_errnum = errno.ESPIPE

    def __init__(self, *args, **kargs):
        self._set_errnum(kargs)
        self._set_frame(kargs)
        self._set_msg(args, kargs)
        super(BuildError, self).__init__(self.msg)

    def _set_errnum(self, kargs):
        if 'errnum' in kargs:
            self.errnum = kargs['errnum']
        elif 'errno' in kargs:
            self.errnum = kargs['errno']
        else:
            self.errnum = self.default_errnum

    def _set_frame(self, kargs):
        if 'frame' in kargs:
            self.frame = kargs['frame']
        else:
            # gof gets a stack of [inner->outer] tuples. tuple[0] is frame
            self.frame = gfi(gof(cf())[2][0])

    def _set_msg(self, args, kargs):
        msg = ''
        frame = self.frame
        if args:
            msg = ': %s' % (', '.join(map(str, args)))
        elif 'message' in kargs:
            msg = kargs['message'] + msg
        elif 'msg' in kargs:
            msg = kargs['msg'] + msg
        else:
            msg = self.default_msg + msg
        self.msg = "(%s) %s [%s:%s]" % (str(self.errnum), msg, frame.filename,
                                        str(frame.lineno))

    def print_msg(self, *args, **kargs):
        """print_msg

A method used often in-code as a means of a catch method.  This method can
accept any number of arguments or key-based arguments; however, it will not
interpret any of them by design.

The function of the method is to print out the exception object's message
attribute.
        """
        print(self.message)


class DebianError(BuildError):
    """DebianError

Raised when there is an issue producing the .deb package for Debian builds.
    """
    default_msg = "An error occurred while building the .deb pkg"

    def __init__(self, *args, **kargs):
        # exception-specific logic here...
        super(DebianError, self).__init__(*args, **kargs)


class RedhatError(BuildError):
    """RedhatError

Raised when there is an issue producing the .rpm package for Redhat builds.
    """
    default_msg = "An error occurred while building the .rpm pkg"

    def __init__(self, *args, **kargs):
        # excpetion-specific logic here...
        super(RedhatError, self).__init__(*args, **kargs)


class TestError(BuildError):
    """TestError

An Error occurred during testing...
    """
    default_msg = "An error occurred during testing phases!"

    def __init__(self, *args, **kargs):
        # exception-specific logic here...
        super(TestError, self).__init__(*args, **kargs)


# vim: set fileencoding=utf-8



# coding=utf-8
#
# Copyright 2017 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#

"""BIG-IP® system management route module

REST URI
    ``http://localhost/mgmt/tm/sys/management-route``

GUI Path
    ``System --> Platform``

REST Kind
    ``tm:sys:management-route:management-routestate``
"""

from distutils.version import LooseVersion
from f5.bigip.resource import Collection
from f5.bigip.resource import Resource


class Management_Routes(Collection):
    """BIG-IP® management-route collection"""
    def __init__(self, sys):
        super(Management_Routes, self).__init__(sys)
        self._meta_data['allowed_lazy_attributes'] = [Management_Route]
        self._meta_data['attribute_registry'] =\
            {'tm:sys:management-route:management-routestate': Management_Route}


class Management_Route(Resource):
    """BIG-IP® system management-route resource"""
    def __init__(self, Management_Routes):
        super(Management_Route, self).__init__(Management_Routes)
        self._meta_data['required_creation_parameters'].update(
            ('network', 'gateway'))
        self._meta_data['required_json_kind'] = \
            'tm:sys:management-route:management-routestate'

    def update(self, **kwargs):
        # Turns out that versions < 12 are incapable of sending the
        # "network" param when modifying. So, to accommodate the ability
        # to update devices in the standard way, we need to pop off the
        # network value on anything < 12.
        #
        # Actual error is this
        #
        # '{
        #   "code":400,
        #   "message":"\\"network\\" may not be specified in the context
        #             of the \\"modify\\" command. \\"network\\" may be
        #             specified using the following commands: create, list",
        #   "errorStack":[]
        # }'
        tmos_v = self._meta_data['bigip']._meta_data['tmos_version']
        if LooseVersion(tmos_v) < LooseVersion('12.0.0'):
            kwargs.pop('network', None)
            self.__dict__.pop('network', None)
        return self._update(**kwargs)

    def modify(self, **kwargs):
        # Turns out that versions < 12 are incapable of sending the
        # "network" param when modifying. So, to accommodate the ability
        # to update devices in the standard way, we need to pop off the
        # network value on anything < 12.
        #
        # Actual error is this
        #
        # '{
        #   "code":400,
        #   "message":"\\"network\\" may not be specified in the context
        #             of the \\"modify\\" command. \\"network\\" may be
        #             specified using the following commands: create, list",
        #   "errorStack":[]
        # }'
        tmos_v = self._meta_data['bigip']._meta_data['tmos_version']
        if LooseVersion(tmos_v) < LooseVersion('12.0.0'):
            kwargs.pop('network', None)
            self.__dict__.pop('network', None)
        return self._modify(**kwargs)



# coding=utf-8
#
# Copyright 2017 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""BIG-IP® system log-config module

REST URI
    ``http://localhost/mgmt/tm/sys/log-config``

GUI Path
    N/A

REST Kind
    ``tm:sys:log-config:*``
"""

from f5.bigip.resource import Collection
from f5.bigip.resource import OrganizingCollection
from f5.bigip.resource import Resource
from f5.sdk_exception import UnsupportedMethod


class Log_Config(OrganizingCollection):
    def __init__(self, sys):
        super(Log_Config, self).__init__(sys)
        self._meta_data['allowed_lazy_attributes'] = [
            Destination,
            Filters,
            Publishers]


class Destination(OrganizingCollection):
    def __init__(self, Log_Config):
        super(Destination, self).__init__(Log_Config)
        self._meta_data['allowed_lazy_attributes'] = [
            Alertds,
            Arcsights,
            Ipfixs,
            Local_Databases,
            Local_Syslogs,
            Management_Ports,
            Remote_High_Speed_Logs,
            Remote_Syslogs,
            Splunks]


class Alertds(Collection):
    def __init__(self, Destination):
        super(Alertds, self).__init__(Destination)
        self._meta_data['allowed_lazy_attributes'] = [Alertd]
        self._meta_data['attribute_registry'] =\
            {'tm:sys:log-config:destination:alertd:alertdstate': Alertd}


class Alertd(Resource):
    def __init__(self, alertds):
        super(Alertd, self).__init__(alertds)
        self._meta_data['required_json_kind'] =\
            'tm:sys:log-config:destination:alertd:alertdstate'

    def create(self, **kwargs):
        '''Create is not supported for Alertd

        :raises: UnsupportedOperation
        '''
        raise UnsupportedMethod(
            "%s does not support the create method" % self.__class__.__name__)

    def delete(self, **kwargs):
        '''Delete is not supported for Alertd

        :raises: UnsupportedOperation
        '''
        raise UnsupportedMethod(
            "%s does not support the delete method" % self.__class__.__name__)


class Arcsights(Collection):
    def __init__(self, Destination):
        super(Arcsights, self).__init__(Destination)
        self._meta_data['allowed_lazy_attributes'] = [Arcsight]
        self._meta_data['attribute_registry'] =\
            {'tm:sys:log-config:destination:arcsight:arcsightstate': Arcsight}


class Arcsight(Resource):
    def __init__(self, arcsights):
        super(Arcsight, self).__init__(arcsights)
        self._meta_data['required_json_kind'] =\
            'tm:sys:log-config:destination:arcsight:arcsightstate'
        self._meta_data['required_creation_parameters'].update(
            ('name', 'forwardTo'))


class Ipfixs(Collection):
    def __init__(self, Destination):
        super(Ipfixs, self).__init__(Destination)
        self._meta_data['allowed_lazy_attributes'] = [Ipfix]
        self._meta_data['attribute_registry'] =\
            {'tm:sys:log-config:destination:ipfix:ipfixstate': Ipfix}


class Ipfix(Resource):
    def __init__(self, ipfixs):
        super(Ipfix, self).__init__(ipfixs)
        self._meta_data['required_json_kind'] =\
            'tm:sys:log-config:destination:ipfix:ipfixstate'
        self._meta_data['required_creation_parameters'].update(
            ('name', 'poolName'))


class Local_Databases(Collection):
    def __init__(self, Destination):
        super(Local_Databases, self).__init__(Destination)
        self._meta_data['allowed_lazy_attributes'] = [Local_Database]
        self._meta_data['attribute_registry'] =\
            {'tm:sys:log-config:destination:local-database:local-databasestate': Local_Database}


class Local_Database(Resource):
    def __init__(self, local_databases):
        super(Local_Database, self).__init__(local_databases)
        self._meta_data['required_json_kind'] =\
            'tm:sys:log-config:destination:local-database:local-databasestate'

    def create(self, **kwargs):
        '''Create is not supported for Local_Database

        :raises: UnsupportedOperation
        '''
        raise UnsupportedMethod(
            "%s does not support the create method" % self.__class__.__name__)

    def delete(self, **kwargs):
        '''Delete is not supported for Local_Database

        :raises: UnsupportedOperation
        '''
        raise UnsupportedMethod(
            "%s does not support the delete method" % self.__class__.__name__)


class Local_Syslogs(Collection):
    def __init__(self, Destination):
        super(Local_Syslogs, self).__init__(Destination)
        self._meta_data['allowed_lazy_attributes'] = [Local_Syslog]
        self._meta_data['attribute_registry'] = \
            {'tm:sys:log-config:destination:local-syslog:local-syslogstate': Local_Syslog}


class Local_Syslog(Resource):
    def __init__(self, local_syslogs):
        super(Local_Syslog, self).__init__(local_syslogs)
        self._meta_data['required_json_kind'] = \
            'tm:sys:log-config:destination:local-syslog:local-syslogstate'

    def create(self, **kwargs):
        '''Create is not supported for Local_Syslog

        :raises: UnsupportedOperation
        '''
        raise UnsupportedMethod(
            "%s does not support the create method" % self.__class__.__name__)

    def delete(self, **kwargs):
        '''Delete is not supported for Local_Syslog

        :raises: UnsupportedOperation
        '''
        raise UnsupportedMethod(
            "%s does not support the delete method" % self.__class__.__name__)


class Management_Ports(Collection):
    def __init__(self, Destination):
        super(Management_Ports, self).__init__(Destination)
        self._meta_data['allowed_lazy_attributes'] = [Management_Port]
        self._meta_data['attribute_registry'] =\
            {'tm:sys:log-config:destination:management-port:management-portstate': Management_Port}


class Management_Port(Resource):
    def __init__(self, management_ports):
        super(Management_Port, self).__init__(management_ports)
        self._meta_data['required_json_kind'] =\
            'tm:sys:log-config:destination:management-port:management-portstate'
        self._meta_data['required_creation_parameters'].update(
            ('name', 'ipAddress', 'port'))


class Remote_High_Speed_Logs(Collection):
    def __init__(self, Destination):
        super(Remote_High_Speed_Logs, self).__init__(Destination)
        self._meta_data['allowed_lazy_attributes'] = [Remote_High_Speed_Log]
        self._meta_data['attribute_registry'] = \
            {'tm:sys:log-config:destination:remote-high-speed-log:remote-high-speed-logstate': Remote_High_Speed_Log}


class Remote_High_Speed_Log(Resource):
    def __init__(self, remote_high_speed_logs):
        super(Remote_High_Speed_Log, self).__init__(remote_high_speed_logs)
        self._meta_data['required_json_kind'] = \
            'tm:sys:log-config:destination:remote-high-speed-log:remote-high-speed-logstate'
        self._meta_data['required_creation_parameters'].update(
            ('name', 'poolName'))


class Remote_Syslogs(Collection):
    def __init__(self, Destination):
        super(Remote_Syslogs, self).__init__(Destination)
        self._meta_data['allowed_lazy_attributes'] = [Remote_Syslog]
        self._meta_data['attribute_registry'] = \
            {'tm:sys:log-config:destination:remote-syslog:remote-syslogstate': Remote_Syslog}


class Remote_Syslog(Resource):
    def __init__(self, remote_syslogs):
        super(Remote_Syslog, self).__init__(remote_syslogs)
        self._meta_data['required_json_kind'] = \
            'tm:sys:log-config:destination:remote-syslog:remote-syslogstate'
        self._meta_data['required_creation_parameters'].update(
            ('name', 'remoteHighSpeedLog'))


class Splunks(Collection):
    def __init__(self, Destination):
        super(Splunks, self).__init__(Destination)
        self._meta_data['allowed_lazy_attributes'] = [Splunk]
        self._meta_data['attribute_registry'] = \
            {'tm:sys:log-config:destination:splunk:splunkstate': Splunk}


class Splunk(Resource):
    def __init__(self, splunks):
        super(Splunk, self).__init__(splunks)
        self._meta_data['required_json_kind'] = \
            'tm:sys:log-config:destination:splunk:splunkstate'
        self._meta_data['required_creation_parameters'].update(
            ('name', 'forwardTo'))


class Filters(Collection):
    def __init__(self, Log_Config):
        super(Filters, self).__init__(Log_Config)
        self._meta_data['allowed_lazy_attributes'] = [Filter]
        self._meta_data['attribute_registry'] =\
            {'tm:sys:log-config:filter:filterstate': Filter}


class Filter(Resource):
    def __init__(self, filters):
        super(Filter, self).__init__(filters)
        self._meta_data['required_json_kind'] =\
            'tm:sys:log-config:filter:filterstate'
        self._meta_data['required_creation_parameters'].update(
            ('name', 'publisher'))


class Publishers(Collection):
    def __init__(self, Log_Config):
        super(Publishers, self).__init__(Log_Config)
        self._meta_data['allowed_lazy_attributes'] = [Publisher]
        self._meta_data['attribute_registry'] =\
            {'tm:sys:log-config:publisher:publisherstate': Publisher}


class Publisher(Resource):
    def __init__(self, publishers):
        super(Publisher, self).__init__(publishers)
        self._meta_data['required_json_kind'] =\
            'tm:sys:log-config:publisher:publisherstate'
        self._meta_data['required_creation_parameters'].update(
            ('name',))



# coding=utf-8
#
# Copyright 2015-2016 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#

from f5.bigip.resource import Collection as BigIpCollection
from f5.bigip.resource import OrganizingCollection as BigIpOrganizingCollection
from f5.bigip.resource import PathElement as BigIpPathElement
from f5.bigip.resource import Resource as BigIpResource
from f5.bigip.resource import ResourceBase as BigIpResourceBase
from f5.bigip.resource import Stats as BigIpStats
from f5.bigip.resource import UnnamedResource as BigIpUnnamedResource


class PathElement(BigIpPathElement):
    """Base class to represent a URI path element that does not contain data.

    The BIG-IP® iControl REST API has URIs that are made up of path components
    that do not return data when they are queried.  This class represents
    those elements and does not support any of the CURDLE methods that
    the other objects do.
    """

    def __init__(self, container):
        super(PathElement, self).__init__(container)
        self._meta_data['minimum_version'] = '2.0.1'


class Resource(BigIpResource, PathElement):
    def __init__(self, container):
        super(Resource, self).__init__(container)
        self._meta_data['required_load_parameters'] = {'uuid', }

    def _assign_stats(self, attrs):
        if self._meta_data['object_has_stats']:
            attrs.append(Stats)
        return attrs


class ResourceBase(BigIpResourceBase, PathElement):
    pass


class OrganizingCollection(BigIpOrganizingCollection, ResourceBase):
    pass


class UnnamedResource(BigIpUnnamedResource, ResourceBase):
    pass


class Collection(BigIpCollection, ResourceBase):
    pass


class Stats(BigIpStats, UnnamedResource):
    pass



# coding=utf-8
#
# Copyright 2017 F5 Networks Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.
#

"""BIG-IP® system management IP module

REST URI
    ``http://localhost/mgmt/tm/sys/management-ip``

GUI Path
    ``System --> Platform``

REST Kind
    ``tm:sys:management-ip:*``
"""

from f5.bigip.resource import Collection
from f5.bigip.resource import Resource


class Management_Ips(Collection):
    """BIG-IP® management-ip collection"""
    def __init__(self, sys):
        super(Management_Ips, self).__init__(sys)
        self._meta_data['allowed_lazy_attributes'] = [Management_Ip]
        self._meta_data['attribute_registry'] =\
            {'tm:sys:management-ip:management-ipstate': Management_Ip}


class Management_Ip(Resource):
    """BIG-IP® management-ip resource"""
    def __init__(self, Management_Ips):
        super(Management_Ip, self).__init__(Management_Ips)
        self._meta_data['required_json_kind'] = \
            'tm:sys:management-ip:management-ipstate'

    def load(self, **kwargs):
        kwargs['transform_name'] = True
        return self._load(**kwargs)



# Copyright 2016 F5 Networks Inc.

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#

from f5.utils.decorators import poll_for_exceptionless_callable


def pollster(callable):
    '''Wraps the poll to get attempts and interval applicable for cluster.

    :param callable: callable -- callable to pass into poll
    '''

    return poll_for_exceptionless_callable(callable, 20, 2)


def get_device_info(bigip):
    '''Get device information about a specific BigIP device.

    :param bigip: ManagementRoot object --- device to inspect
    :returns: ManagementRoot object
    '''

    coll = pollster(bigip.tm.cm.devices.get_collection)()
    device = [device for device in coll if device.selfDevice == 'true']
    assert len(device) == 1
    return device[0]


def get_device_names_to_objects(devices):
    '''Map a list of devices to their hostnames.

    :param devices: list -- list of ManagementRoot objects
    :returns: dict -- mapping of hostnames to ManagementRoot objects
    '''

    name_to_object = {}
    for device in devices:
        device_name = get_device_info(device).name
        name_to_object[device_name] = device
    return name_to_object
